#!/usr/bin/python
# Copyright (c) 2017, Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ipa_role
author: Thomas Krahn (@Nosmoht)
short_description: Manage FreeIPA role
description:
  - Add, modify and delete a role within FreeIPA server using FreeIPA API.
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  cn:
    description:
      - Role name.
      - Can not be changed as it is the unique identifier.
    required: true
    aliases: ['name']
    type: str
  description:
    description:
      - A description of this role-group.
    type: str
  group:
    description:
      - List of group names assign to this role.
      - If an empty list is passed all assigned groups are unassigned from the role.
      - If option is omitted groups are not checked nor changed.
      - If option is passed all assigned groups that are not passed are unassigned from the role.
    type: list
    elements: str
  host:
    description:
      - List of host names to assign.
      - If an empty list is passed all assigned hosts are unassigned from the role.
      - If option is omitted hosts are not checked nor changed.
      - If option is passed all assigned hosts that are not passed are unassigned from the role.
    type: list
    elements: str
  hostgroup:
    description:
      - List of host group names to assign.
      - If an empty list is passed all assigned host groups are removed from the role.
      - If option is omitted host groups are not checked nor changed.
      - If option is passed all assigned hostgroups that are not passed are unassigned from the role.
    type: list
    elements: str
  privilege:
    description:
      - List of privileges granted to the role.
      - If an empty list is passed all assigned privileges are removed.
      - If option is omitted privileges are not checked nor changed.
      - If option is passed all assigned privileges that are not passed are removed.
    type: list
    elements: str
  service:
    description:
      - List of service names to assign.
      - If an empty list is passed all assigned services are removed from the role.
      - If option is omitted services are not checked nor changed.
      - If option is passed all assigned services that are not passed are removed from the role.
    type: list
    elements: str
  state:
    description: State to ensure.
    default: "present"
    choices: ["absent", "present"]
    type: str
  user:
    description:
      - List of user names to assign.
      - If an empty list is passed all assigned users are removed from the role.
      - If option is omitted users are not checked nor changed.
    type: list
    elements: str
extends_documentation_fragment:
  - community.general._ipa.documentation
  - community.general._ipa.connection_notes
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Ensure role is present
  community.general.ipa_role:
    name: dba
    description: Database Administrators
    state: present
    user:
      - pinky
      - brain
    ipa_host: ipa.example.com
    ipa_user: admin
    ipa_pass: topsecret

- name: Ensure role with certain details
  community.general.ipa_role:
    name: another-role
    description: Just another role
    group:
      - editors
    host:
      - host01.example.com
    hostgroup:
      - hostgroup01
    privilege:
      - Group Administrators
      - User Administrators
    service:
      - service01

- name: Ensure role is absent
  community.general.ipa_role:
    name: dba
    state: absent
    ipa_host: ipa.example.com
    ipa_user: admin
    ipa_pass: topsecret
"""

RETURN = r"""
role:
  description: Role as returned by IPA API.
  returned: always
  type: dict
"""

import traceback

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._ipa import IPAClient, ipa_argument_spec


class RoleIPAClient(IPAClient):
    def __init__(self, module, host, port, protocol):
        super().__init__(module, host, port, protocol)

    def role_find(self, name):
        return self._post_json(method="role_find", name=None, item={"all": True, "cn": name})

    def role_add(self, name, item):
        return self._post_json(method="role_add", name=name, item=item)

    def role_mod(self, name, item):
        return self._post_json(method="role_mod", name=name, item=item)

    def role_del(self, name):
        return self._post_json(method="role_del", name=name)

    def role_add_member(self, name, item):
        return self._post_json(method="role_add_member", name=name, item=item)

    def role_add_group(self, name, item):
        return self.role_add_member(name=name, item={"group": item})

    def role_add_host(self, name, item):
        return self.role_add_member(name=name, item={"host": item})

    def role_add_hostgroup(self, name, item):
        return self.role_add_member(name=name, item={"hostgroup": item})

    def role_add_service(self, name, item):
        return self.role_add_member(name=name, item={"service": item})

    def role_add_user(self, name, item):
        return self.role_add_member(name=name, item={"user": item})

    def role_remove_member(self, name, item):
        return self._post_json(method="role_remove_member", name=name, item=item)

    def role_remove_group(self, name, item):
        return self.role_remove_member(name=name, item={"group": item})

    def role_remove_host(self, name, item):
        return self.role_remove_member(name=name, item={"host": item})

    def role_remove_hostgroup(self, name, item):
        return self.role_remove_member(name=name, item={"hostgroup": item})

    def role_remove_service(self, name, item):
        return self.role_remove_member(name=name, item={"service": item})

    def role_remove_user(self, name, item):
        return self.role_remove_member(name=name, item={"user": item})

    def role_add_privilege(self, name, item):
        return self._post_json(method="role_add_privilege", name=name, item={"privilege": item})

    def role_remove_privilege(self, name, item):
        return self._post_json(method="role_remove_privilege", name=name, item={"privilege": item})


def get_role_dict(description=None):
    data = {}
    if description is not None:
        data["description"] = description
    return data


def get_role_diff(client, ipa_role, module_role):
    return client.get_diff(ipa_data=ipa_role, module_data=module_role)


def ensure(module, client):
    state = module.params["state"]
    name = module.params["cn"]
    group = module.params["group"]
    host = module.params["host"]
    hostgroup = module.params["hostgroup"]
    privilege = module.params["privilege"]
    service = module.params["service"]
    user = module.params["user"]

    module_role = get_role_dict(description=module.params["description"])
    ipa_role = client.role_find(name=name)

    changed = False
    if state == "present":
        if not ipa_role:
            changed = True
            if not module.check_mode:
                ipa_role = client.role_add(name=name, item=module_role)
        else:
            diff = get_role_diff(client, ipa_role, module_role)
            if len(diff) > 0:
                changed = True
                if not module.check_mode:
                    data = {}
                    for key in diff:
                        data[key] = module_role.get(key)
                    client.role_mod(name=name, item=data)

        if group is not None:
            changed = (
                client.modify_if_diff(
                    name, ipa_role.get("member_group", []), group, client.role_add_group, client.role_remove_group
                )
                or changed
            )
        if host is not None:
            changed = (
                client.modify_if_diff(
                    name, ipa_role.get("member_host", []), host, client.role_add_host, client.role_remove_host
                )
                or changed
            )

        if hostgroup is not None:
            changed = (
                client.modify_if_diff(
                    name,
                    ipa_role.get("member_hostgroup", []),
                    hostgroup,
                    client.role_add_hostgroup,
                    client.role_remove_hostgroup,
                )
                or changed
            )

        if privilege is not None:
            changed = (
                client.modify_if_diff(
                    name,
                    ipa_role.get("memberof_privilege", []),
                    privilege,
                    client.role_add_privilege,
                    client.role_remove_privilege,
                )
                or changed
            )
        if service is not None:
            changed = (
                client.modify_if_diff(
                    name,
                    ipa_role.get("member_service", []),
                    service,
                    client.role_add_service,
                    client.role_remove_service,
                )
                or changed
            )
        if user is not None:
            changed = (
                client.modify_if_diff(
                    name, ipa_role.get("member_user", []), user, client.role_add_user, client.role_remove_user
                )
                or changed
            )

    else:
        if ipa_role:
            changed = True
            if not module.check_mode:
                client.role_del(name)

    return changed, client.role_find(name=name)


def main():
    argument_spec = ipa_argument_spec()
    argument_spec.update(
        cn=dict(type="str", required=True, aliases=["name"]),
        description=dict(type="str"),
        group=dict(type="list", elements="str"),
        host=dict(type="list", elements="str"),
        hostgroup=dict(type="list", elements="str"),
        privilege=dict(type="list", elements="str"),
        service=dict(type="list", elements="str"),
        state=dict(type="str", default="present", choices=["present", "absent"]),
        user=dict(type="list", elements="str"),
    )

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    client = RoleIPAClient(
        module=module,
        host=module.params["ipa_host"],
        port=module.params["ipa_port"],
        protocol=module.params["ipa_prot"],
    )

    try:
        client.login(username=module.params["ipa_user"], password=module.params["ipa_pass"])
        changed, role = ensure(module, client)
        module.exit_json(changed=changed, role=role)
    except Exception as e:
        module.fail_json(msg=f"{e}", exception=traceback.format_exc())


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Shreyas Srish (@shrsr) <ssrish@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_ptp_policy
short_description: Manage PTP Policies on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage Precision Time Protocol (PTP) Policies on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Shreyas Srish (@shrsr)
options:
  template:
    description:
    - The name of the template.
    - The template must be a fabric policy template.
    type: str
    required: true
  ptp_policy:
    description:
    - The name of the PTP policy.
    type: str
    aliases: [ name ]
  ptp_policy_uuid:
    description:
    - The UUID of the PTP policy.
    - This parameter is required when the O(ptp_policy) needs to be updated.
    type: str
    aliases: [ uuid ]
  description:
    description:
    - The description of the PTP Policy.
    type: str
  admin_state:
    description:
    - The administrative state of the PTP Policy.
    type: str
    choices: [ enabled, disabled ]
  fabric_sync_interval:
    description:
    - Fabric synchronization interval for the PTP Policy.
    - The value should be between -4 to 1.
    - This is required when O(state=present).
    type: int
  global_domain:
    description:
    - Global domain number for the PTP Policy.
    - The value should be between 1 to 128.
    - This is required when O(state=present).
    type: int
  fabric_delay_interval:
    description:
    - Fabric delay interval for the PTP policy.
    - The value should be between -3 to 5.
    - This is required when O(state=present).
    type: int
  global_priority1:
    description:
    - Priority 1 for the PTP Policy.
    - The value should be between 1 to 255.
    - This is required when O(state=present).
    type: int
  global_priority2:
    description:
    - Priority 2 for the PTP Policy.
    - The value should be between 1 to 255.
    - This is required when O(state=present).
    type: int
  fabric_announce_timeout:
    description:
    - Fabric announce timeout for the PTP policy.
    - The value should be between 2 to 10.
    - This is required when O(state=present).
    type: int
  fabric_announce_interval:
    description:
    - Fabric announce interval for the PTP policy.
    - The value should be between 0 to 4.
    - This is required when O(state=present).
    type: int
  fabric_profile_template:
    description:
    - Fabric profile template for the PTP policy.
    type: str
    choices: [ aes67_2015, default, smpte_2059_2 ]
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  The M(cisco.mso.ndo_template) module can be used for this.
seealso:
- module: cisco.mso.ndo_template
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a new PTP policy
  cisco.mso.ndo_ptp_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_fabric_policy_template
    ptp_policy: ansible_test_ptp_policy
    admin_state: enabled
    fabric_sync_interval: -3
    global_domain: 0
    fabric_delay_interval: -2
    global_priority1: 255
    global_priority2: 255
    fabric_announce_timeout: 3
    fabric_announce_interval: 1
    fabric_profile_template: aes67_2015
    state: present
  register: create_ptp

- name: Query a PTP policy with ptp_policy name
  cisco.mso.ndo_ptp_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_fabric_policy_template
    ptp_policy: ansible_test_ptp_policy
    state: query
  register: query_one

- name: Query a PTP policy with ptp_policy UUID
  cisco.mso.ndo_ptp_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_fabric_policy_template
    ptp_policy_uuid: "{{ create_ptp.current.uuid }}"
    state: query
  register: query_one_uuid

- name: Update a PTP policy name with UUID
  cisco.mso.ndo_ptp_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    ptp_policy: ansible_test_ptp_policy_updated
    ptp_policy_uuid: "{{ create_ptp.current.uuid }}"
    state: present

- name: Delete a PTP policy
  cisco.mso.ndo_ptp_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_ptp_policy_updated
    ptp_policy: ansible_test_ptp_policy
    state: absent

- name: Delete a PTP policy using UUID
  cisco.mso.ndo_ptp_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_fabric_policy_template
    ptp_policy_uuid: "{{ query_one.current.uuid }}"
    state: absent
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate, KVPair
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data
from ansible_collections.cisco.mso.plugins.module_utils.constants import PROFILE_TEMPLATE
import copy


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        dict(
            template=dict(type="str", required=True),
            ptp_policy=dict(type="str", aliases=["name"]),
            ptp_policy_uuid=dict(type="str", aliases=["uuid"]),
            description=dict(type="str"),
            admin_state=dict(type="str", choices=["enabled", "disabled"]),
            fabric_sync_interval=dict(type="int"),
            global_domain=dict(type="int"),
            fabric_delay_interval=dict(type="int"),
            global_priority1=dict(type="int"),
            global_priority2=dict(type="int"),
            fabric_announce_timeout=dict(type="int"),
            fabric_announce_interval=dict(type="int"),
            fabric_profile_template=dict(type="str", choices=["aes67_2015", "default", "smpte_2059_2"]),
            state=dict(type="str", choices=["absent", "query", "present"], default="query"),
        )
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "present", ["ptp_policy", "ptp_policy_uuid"], True],
            ["state", "absent", ["ptp_policy", "ptp_policy_uuid"], True],
            [
                "state",
                "present",
                [
                    "global_priority1",
                    "global_priority2",
                    "global_domain",
                    "fabric_announce_timeout",
                    "fabric_announce_interval",
                    "fabric_sync_interval",
                    "fabric_delay_interval",
                ],
            ],
        ],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    ptp_policy = module.params.get("ptp_policy")
    ptp_policy_uuid = module.params.get("ptp_policy_uuid")
    description = module.params.get("description")
    admin_state = module.params.get("admin_state")
    fabric_sync_interval = module.params.get("fabric_sync_interval")
    global_domain = module.params.get("global_domain")
    fabric_delay_interval = module.params.get("fabric_delay_interval")
    global_priority1 = module.params.get("global_priority1")
    global_priority2 = module.params.get("global_priority2")
    fabric_announce_timeout = module.params.get("fabric_announce_timeout")
    fabric_announce_interval = module.params.get("fabric_announce_interval")
    fabric_profile_template = PROFILE_TEMPLATE.get(module.params.get("fabric_profile_template"))
    state = module.params.get("state")

    ops = []
    match = None

    mso_template = MSOTemplate(mso, "fabric_policy", template)
    mso_template.validate_template("fabricPolicy")
    object_description = "PTP Policy"

    path = "/fabricPolicyTemplate/template/ptpPolicy"

    existing_ptp_policies = mso_template.template.get("fabricPolicyTemplate", {}).get("template", {}).get("ptpPolicy", {})

    if ptp_policy or ptp_policy_uuid:
        match = mso_template.get_object_by_key_value_pairs(
            object_description,
            [existing_ptp_policies],
            [KVPair("uuid", ptp_policy_uuid) if ptp_policy_uuid else KVPair("name", ptp_policy)],
        )
        if match:
            mso.existing = mso.previous = copy.deepcopy(match.details)
    else:
        mso.existing = mso.previous = existing_ptp_policies

    if state == "present":
        if match:
            mso_values_update = {
                "name": ptp_policy,
                "description": description,
                ("global", "adminState"): admin_state,
                ("global", "fabSyncIntvl"): fabric_sync_interval,
                ("global", "globalDomain"): global_domain,
                ("global", "fabDelayIntvl"): fabric_delay_interval,
                ("global", "prio1"): global_priority1,
                ("global", "prio2"): global_priority2,
                ("global", "fabAnnounceTimeout"): fabric_announce_timeout,
                ("global", "fabAnnounceIntvl"): fabric_announce_interval,
                ("global", "fabProfileTemplate"): fabric_profile_template,
            }
            append_update_ops_data(ops, match.details, path, mso_values_update)
            mso.sanitize(match.details, collate=True)
        else:
            mso_values = {
                "name": ptp_policy,
                "description": description,
                "global": {
                    "adminState": admin_state,
                    "fabSyncIntvl": fabric_sync_interval,
                    "globalDomain": global_domain,
                    "fabDelayIntvl": fabric_delay_interval,
                    "prio1": global_priority1,
                    "prio2": global_priority2,
                    "fabAnnounceTimeout": fabric_announce_timeout,
                    "fabAnnounceIntvl": fabric_announce_interval,
                    "fabProfileTemplate": fabric_profile_template,
                },
            }
            mso.sanitize(mso_values)
            ops.append(dict(op="add", path="{0}".format(path), value=mso.sent))

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path="{0}".format(path)))

    if not module.check_mode and ops:
        response = mso.request(mso_template.template_path, method="PATCH", data=ops)
        ptp_policies = response.get("fabricPolicyTemplate", {}).get("template", {}).get("ptpPolicy", {})
        match = mso_template.get_object_by_key_value_pairs(
            object_description,
            [ptp_policies],
            [KVPair("uuid", ptp_policy_uuid) if ptp_policy_uuid else KVPair("name", ptp_policy)],
        )
        if match:
            mso.existing = match.details
        else:
            mso.existing = {}
    elif module.check_mode and state != "query":
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
"""
The module file for ios_lacp
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: ios_lacp
short_description: Resource module to configure LACP.
description:
  This module provides declarative management of Global LACP on Cisco IOS
  network devices.
version_added: 1.0.0
author: Sumit Jaiswal (@justjais)
notes:
  - Tested against Cisco IOSXE Version 17.3 on CML.
  - This module works with connection C(network_cli).
    See U(https://docs.ansible.com/ansible/latest/network/user_guide/platform_ios.html)
options:
  config:
    description: The provided configurations.
    type: dict
    suboptions:
      system:
        description: This option sets the default system parameters for LACP.
        type: dict
        suboptions:
          priority:
            description:
              - LACP priority for the system.
              - Refer to vendor documentation for valid values.
            type: int
            required: true
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the IOS device by
        executing the command B(show lacp sys-id).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    description:
      - The state the configuration should be left in
      - The module have declaratively similar behavior for replaced and overridden state.
      - The states I(rendered), I(gathered) and I(parsed) does not perform any change
        on the device.
      - The state I(rendered) will transform the configuration in C(config) option to
        platform specific CLI commands which will be returned in the I(rendered) key
        within the result. For state I(rendered) active connection to remote host is
        not required.
      - The state I(gathered) will fetch the running configuration from device and transform
        it into structured data in the format as per the resource module argspec and
        the value is returned in the I(gathered) key within the result.
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into JSON format as per the resource module parameters and the
        value is returned in the I(parsed) key within the result. The value of C(running_config)
        option should be the same format as the output of command I(show running-config
        | include ip route|ipv6 route) executed on device. For state I(parsed) active
        connection to remote host is not required.
    type: str
    choices:
      - merged
      - replaced
      - overridden
      - deleted
      - rendered
      - parsed
      - gathered
    default: merged
"""

EXAMPLES = """
# Using merged
#
# Before state:
# -------------
#
# vios#show lacp sys-id
# 32768, 5e00.0000.8000

- name: Merge provided configuration with device configuration
  cisco.ios.ios_lacp:
    config:
      system:
        priority: 123
    state: merged

# After state:
# ------------
#
# vios#show lacp sys-id
# 123, 5e00.0000.8000

# Using replaced
#
# Before state:
# -------------
#
# vios#show lacp sys-id
# 500, 5e00.0000.8000

- name: Replaces Global LACP configuration
  cisco.ios.ios_lacp:
    config:
      system:
        priority: 123
    state: replaced

# After state:
# ------------
#
# vios#show lacp sys-id
# 123, 5e00.0000.8000

# Using Deleted
#
# Before state:
# -------------
#
# vios#show lacp sys-id
# 500, 5e00.0000.8000

- name: Delete Global LACP attribute
  cisco.ios.ios_lacp:
    state: deleted

# After state:
# -------------
#
# vios#show lacp sys-id
# 32768, 5e00.0000.8000

# Using Gathered

# Before state:
# -------------
#
# vios#show lacp sys-id
# 123, 5e00.0000.8000

- name: Gather listed LACP with provided configurations
  cisco.ios.ios_lacp:
    config:
    state: gathered

# Module Execution Result:
# ------------------------
#
# "gathered": {
#         "system": {
#             "priority": 500
#         }
#     }

# After state:
# ------------
#
# vios#show lacp sys-id
# 123, 5e00.0000.8000

# Using Rendered

- name: Render the commands for provided  configuration
  cisco.ios.ios_lacp:
    config:
      system:
        priority: 123
    state: rendered

# Module Execution Result:
# ------------------------
#
# "rendered": [
#         "lacp system-priority 10"
#     ]

# Using Parsed

# File: parsed.cfg
# ----------------
#
# lacp system-priority 123

- name: Parse the commands for provided configuration
  cisco.ios.ios_lacp:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Module Execution Result:
# ------------------------
#
# "parsed": {
#         "system": {
#             "priority": 123
#         }
#     }
"""

RETURN = """
before:
  description: The configuration as structured data prior to module invocation.
  returned: always
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The configuration as structured data after module completion.
  returned: when changed
  type: list
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample: ['lacp system-priority 10']
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.lacp.lacp import (
    LacpArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.config.lacp.lacp import Lacp


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    required_if = [
        ("state", "merged", ("config",)),
        ("state", "replaced", ("config",)),
        ("state", "overridden", ("config",)),
        ("state", "rendered", ("config",)),
        ("state", "parsed", ("running_config",)),
    ]

    mutually_exclusive = [("config", "running_config")]

    module = AnsibleModule(
        argument_spec=LacpArgs.argument_spec,
        required_if=required_if,
        mutually_exclusive=mutually_exclusive,
        supports_check_mode=True,
    )
    result = Lacp(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_live_tools_cable_test_info
short_description: Information module for devices _live _tools _cable _test
description: >
  Devices Live Tools Cable Test Info.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
notes:
  - Paths used are
"""

EXAMPLES = r"""
- name: Get devices_live_tools_cable_test_info info
  cisco.meraki.devices_live_tools_cable_test_info:
    meraki_api_key: "{{ansible_meraki_api_key}}"
    state: query
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample:
  - {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Ansible module to manage CheckPoint Firewall (c) 2019
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: cp_mgmt_vsx_run_operation
short_description: Run the VSX operation by its name and parameters.
description:
  - Run the VSX operation by its name and parameters.
  - An automatic session publish is part of all the operations in this API.
  - All operations are performed over Web Services API.
  - Available from R81 management version.
version_added: "5.0.0"
author: "Eden Brillant (@chkp-edenbr)"
options:
  operation:
    description:
      - The name of the operation to run. Each operation has its specific parameters.<br>The available operations are,<ul><li><i>upgrade</i> -
        Upgrades the VSX Gateway or VSX Cluster object to a higher version</li><li><i>downgrade</i> - Downgrades the VSX Gateway or VSX Cluster object to a
        lower version</li><li><i>add-member</i> - Adds a new VSX Cluster member object</li><li><i>remove-member</i> - Removes a VSX Cluster member
        object</li><li><i>reconf-gw</i> - Reconfigures a VSX Gateway after a clean install</li><li><i>reconf-member</i> - Reconfigures a VSX Cluster member
        after a clean install</li></ul>.
    type: str
    choices: ['upgrade', 'downgrade', 'add-member', 'remove-member', 'reconf-gw', 'reconf-member']
  add_member_params:
    description:
      - Parameters for the operation to add a VSX Cluster member.
    type: dict
    suboptions:
      ipv4_address:
        description:
          - The IPv4 address of the management interface of the VSX Cluster member.
        type: str
      ipv4_sync_address:
        description:
          - The IPv4 address of the sync interface of the VSX Cluster member.
        type: str
      member_name:
        description:
          - Name of the new VSX Cluster member object.
        type: str
      vsx_name:
        description:
          - Name of the VSX Cluster object.
        type: str
      vsx_uid:
        description:
          - UID of the VSX Cluster object.
        type: str
  downgrade_params:
    description:
      - Parameters for the operation to downgrade a VSX Gateway or VSX Cluster object to a lower version.<br>In case the current version is already
        the target version, or is lower than the target version, no change is done.
    type: dict
    suboptions:
      target_version:
        description:
          - The target version.
        type: str
      vsx_name:
        description:
          - Name of the VSX Gateway or VSX Cluster object.
        type: str
      vsx_uid:
        description:
          - UID of the VSX Gateway or VSX Cluster object.
        type: str
  reconf_gw_params:
    description:
      - Parameters for the operation to reconfigure a VSX Gateway after a clean install.
    type: dict
    suboptions:
      ipv4_corexl_number:
        description:
          - Number of IPv4 CoreXL Firewall instances on the target VSX Gateway.<br>Valid values,<br><ul><li>To configure CoreXL Firewall
            instances, enter an integer greater or equal to 2.</li><li>To disable CoreXL, enter 1.</li></ul>.
        type: int
      one_time_password:
        description:
          - A password required for establishing a Secure Internal Communication (SIC). Enter the same password you used during the First Time
            Configuration Wizard on the target VSX Gateway.
        type: str
      vsx_name:
        description:
          - Name of the VSX Gateway object.
        type: str
      vsx_uid:
        description:
          - UID of the VSX Gateway object.
        type: str
  reconf_member_params:
    description:
      - Parameters for the operation to reconfigure a VSX Cluster member after a clean install.
    type: dict
    suboptions:
      ipv4_corexl_number:
        description:
          - Number of IPv4 CoreXL Firewall instances on the target VSX Cluster member.<br>Valid values,<br><ul><li>To configure CoreXL Firewall
            instances, enter an integer greater or equal to 2.</li><li>To disable CoreXL, enter 1.</li></ul>Important - The CoreXL configuration must be the
            same on all the cluster members.
        type: int
      member_uid:
        description:
          - UID of the VSX Cluster member object.
        type: str
      member_name:
        description:
          - Name of the VSX Cluster member object.
        type: str
      one_time_password:
        description:
          - A password required for establishing a Secure Internal Communication (SIC). Enter the same password you used during the First Time
            Configuration Wizard on the target VSX Cluster member.
        type: str
  remove_member_params:
    description:
      - Parameters for the operation to remove a VSX Cluster member object.
    type: dict
    suboptions:
      member_uid:
        description:
          - UID of the VSX Cluster member object.
        type: str
      member_name:
        description:
          - Name of the VSX Cluster member object.
        type: str
  upgrade_params:
    description:
      - Parameters for the operation to upgrade a VSX Gateway or VSX Cluster object to a higher version.<br>In case the current version is already the
        target version, or is higher than the target version, no change is done.
    type: dict
    suboptions:
      target_version:
        description:
          - The target version.
        type: str
      vsx_name:
        description:
          - Name of the VSX Gateway or VSX Cluster object.
        type: str
      vsx_uid:
        description:
          - UID of the VSX Gateway or VSX Cluster object.
        type: str
extends_documentation_fragment: check_point.mgmt.checkpoint_commands
"""

EXAMPLES = """
- name: vsx-run-operation
  cp_mgmt_vsx_run_operation:
    add_member_params:
      ipv4_address: 25.25.25.223
      ipv4_sync_address: 20.20.20.223
      member_name: Mem3
      vsx_name: VSX_CLUSTER
    operation: add-member
"""

RETURN = """
cp_mgmt_vsx_run_operation:
  description: The checkpoint vsx-run-operation output.
  returned: always.
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.check_point.mgmt.plugins.module_utils.checkpoint import checkpoint_argument_spec_for_commands, api_command


def main():
    argument_spec = dict(
        operation=dict(type='str', choices=['upgrade', 'downgrade', 'add-member', 'remove-member', 'reconf-gw', 'reconf-member']),
        add_member_params=dict(type='dict', options=dict(
            ipv4_address=dict(type='str'),
            ipv4_sync_address=dict(type='str'),
            member_name=dict(type='str'),
            vsx_name=dict(type='str'),
            vsx_uid=dict(type='str')
        )),
        downgrade_params=dict(type='dict', options=dict(
            target_version=dict(type='str'),
            vsx_name=dict(type='str'),
            vsx_uid=dict(type='str')
        )),
        reconf_gw_params=dict(type='dict', options=dict(
            ipv4_corexl_number=dict(type='int'),
            one_time_password=dict(type='str', no_log=True),
            vsx_name=dict(type='str'),
            vsx_uid=dict(type='str')
        )),
        reconf_member_params=dict(type='dict', options=dict(
            ipv4_corexl_number=dict(type='int'),
            member_uid=dict(type='str'),
            member_name=dict(type='str'),
            one_time_password=dict(type='str', no_log=True)
        )),
        remove_member_params=dict(type='dict', options=dict(
            member_uid=dict(type='str'),
            member_name=dict(type='str')
        )),
        upgrade_params=dict(type='dict', options=dict(
            target_version=dict(type='str'),
            vsx_name=dict(type='str'),
            vsx_uid=dict(type='str')
        ))
    )
    argument_spec.update(checkpoint_argument_spec_for_commands)

    module = AnsibleModule(argument_spec=argument_spec)

    command = "vsx-run-operation"

    result = api_command(module, command)
    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: lambda_info
version_added: 5.0.0
short_description: Gathers AWS Lambda function details
description:
  - Gathers various details related to Lambda functions, including aliases, versions and event source mappings.
  - Use module M(amazon.aws.lambda) to manage the lambda function itself, M(amazon.aws.lambda_alias) to manage function aliases,
    M(amazon.aws.lambda_event) to manage lambda event source mappings, and M(amazon.aws.lambda_policy) to manage policy statements.
  - This module was originally added to C(community.aws) in release 1.0.0.
options:
  query:
    description:
      - Specifies the resource type for which to gather information.
      - Defaults to V(all) when O(function_name) is specified.
      - Defaults to V(config) when O(function_name) is NOT specified.
    choices: [ "aliases", "all", "config", "mappings", "policy", "versions", "tags" ]
    type: str
  function_name:
    description:
      - The name of the lambda function for which information is requested.
    aliases: [ "function", "name"]
    type: str
  event_source_arn:
    description:
      - When O(query=mappings), this is the Amazon Resource Name (ARN) of the Amazon Kinesis or DynamoDB stream.
    type: str
author:
  - Pierre Jodouin (@pjodouin)
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
---
# Simple example of listing all info for a function
- name: List all for a specific function
  amazon.aws.lambda_info:
    query: all
    function_name: myFunction
  register: my_function_details

# Get basic information about AWS Lambda Functions in the current region
- name: List Lambda Functions
  amazon.aws.lambda_info:
  register: lambda_functions

# List all versions of a function
- name: List function versions
  amazon.aws.lambda_info:
    query: versions
    function_name: myFunction
  register: my_function_versions

# List all info for all functions
- name: List all functions
  amazon.aws.lambda_info:
    query: all
  register: output

- name: show Lambda information
  ansible.builtin.debug:
    msg: "{{ output['function'] }}"
"""

RETURN = r"""
---
functions:
    description: List of information for each lambda function matching the query.
    returned: always
    type: list
    elements: dict
    version_added: 4.1.0
    version_added_collection: community.aws
    contains:
        aliases:
            description: The aliases associated with the function.
            returned: when O(query=aliases) or O(query=all)
            type: list
            elements: str
        architectures:
            description: The architectures supported by the function.
            returned: success
            type: list
            elements: str
            sample: ['arm64']
        code_sha256:
            description: The SHA256 hash of the function's deployment package.
            returned: success
            type: str
            sample: 'zOAGfF5JLFuzZoSNirUtOrQp+S341IOA3BcoXXoaIaU='
        code_size:
            description: The size of the function's deployment package in bytes.
            returned: success
            type: int
            sample: 123
        dead_letter_config:
            description: The function's dead letter queue.
            returned: when the function has a dead letter queue configured
            type: dict
            sample: { 'target_arn': arn:aws:lambda:us-east-1:123456789012:function:myFunction:1 }
            contains:
                target_arn:
                    description: The ARN of an SQS queue or SNS topic.
                    returned: when the function has a dead letter queue configured
                    type: str
                    sample: arn:aws:lambda:us-east-1:123456789012:function:myFunction:1
        description:
            description: The function's description.
            returned: success
            type: str
            sample: 'My function'
        ephermal_storage:
            description: The size of the function /tmp directory in MB.
            type: dict
            contains:
                size:
                    description: The size of the function /tmp directory.
                    type: int
                    returned: always
                    sample: 512
        environment:
            description: The function's environment variables.
            returned: when environment variables exist
            type: dict
            contains:
                variables:
                    description: Environment variable key-value pairs.
                    returned: when environment variables exist
                    type: dict
                    sample: {'key': 'value'}
                error:
                    description: Error message for environment variables that could not be applied.
                    returned: when there is an error applying environment variables
                    type: dict
                    contains:
                        error_code:
                            description: The error code.
                            returned: when there is an error applying environment variables
                            type: str
                        message:
                            description: The error message.
                            returned: when there is an error applying environment variables
                            type: str
        function_arn:
            description: The function's Amazon Resource Name (ARN).
            returned: on success
            type: str
            sample: 'arn:aws:lambda:us-east-1:123456789012:function:myFunction:1'
        function_name:
            description: The function's name.
            returned: on success
            type: str
            sample: 'myFunction'
        handler:
            description: The function Lambda calls to begin executing your function.
            returned: on success
            type: str
            sample: 'index.handler'
        last_modified:
            description: The date and time that the function was last updated, in ISO-8601 format (YYYY-MM-DDThh:mm:ssTZD).
            returned: on success
            type: str
            sample: '2017-08-01T00:00:00.000+0000'
        mappings:
            description: List of configuration information for each event source mapping.
            returned: when O(query=all) or O(query=mappings)
            type: list
            elements: dict
            contains:
                uuid:
                    description: The AWS Lambda assigned opaque identifier for the mapping.
                    returned: on success
                    type: str
                batch_size:
                    description: The largest number of records that AWS Lambda will retrieve from the event source at the time of invoking the function.
                    returned: on success
                    type: int
                event_source_arn:
                    description: The ARN of the Amazon Kinesis or DyanmoDB stream that is the source of events.
                    returned: on success
                    type: str
                function_arn:
                    description: The Lambda function to invoke when AWS Lambda detects an event on the poll-based source.
                    returned: on success
                    type: str
                last_modified:
                    description: The UTC time string indicating the last time the event mapping was updated.
                    returned: on success
                    type: str
                last_processing_result:
                    description: The result of the last AWS Lambda invocation of your Lambda function.
                    returned: on success
                    type: str
                state:
                    description: The state of the event source mapping.
                    returned: on success
                    type: str
                state_transition_reason:
                    description: The reason the event source mapping is in its current state.
                    returned: on success
                    type: str
        memory_size:
            description: The memory allocated to the function.
            returned: on success
            type: int
            sample: 128
        package_type:
            description: The type of deployment package.
            type: str
            returned: always
            sample: "Zip"
        policy:
            description: The policy associated with the function.
            returned: when O(query=all) or O(query=policy)
            type: dict
        revision_id:
            description: The latest updated revision of the function or alias.
            returned: on success
            type: str
            sample: 'a2x9886d-d48a-4a0c-ab64-82abc005x80c'
        role:
            description: The function's execution role.
            returned: on success
            type: str
            sample: 'arn:aws:iam::123456789012:role/lambda_basic_execution'
        runtime:
            description: The funtime environment for the Lambda function.
            returned: on success
            type: str
            sample: 'nodejs6.10'
        runtime_version_config:
            description: The ARN of the runtime and any errors that occured.
            type: dict
            contains:
                runtime_version_arn:
                    description: The ARN of the runtime version you want the function to use.
                    type: str
                    returned: always
                    sample: "arn:aws:lambda:us-west-2::runtime:123456789012"
        snap_start:
            description: Snapshot of the initialized execution environment when you publish a function version.
            type: dict
            contains:
                apply_on:
                    description: When set to V(PublishedVersions), Lambda creates a snapshot of the execution environment when you publish a function version.
                    type: str
                    sample: "None"
                    returned: always
                optimization_status:
                    description: This element indicates whether SnapStart is activated for the specified function version.
                    type: str
                    returned: always
                    sample: "Off"
        tracing_config:
            description: The function's AWS X-Ray tracing configuration.
            returned: on success
            type: dict
            sample: { 'mode': 'Active' }
            contains:
                mode:
                    description: The tracing mode.
                    returned: on success
                    type: str
                    sample: 'Active'
        timeout:
            description: The amount of time that Lambda allows a function to run before terminating it.
            returned: on success
            type: int
            sample: 3
        version:
            description: The version of the Lambda function.
            returned: on success
            type: str
            sample: '1'
        versions:
            description: List of Lambda function versions.
            returned: when O(query=all) or O(query=versions)
            type: list
            elements: dict
        vpc_config:
            description: The function's networking configuration.
            returned: on success
            type: dict
            sample: {
            'security_group_ids': [],
            'subnet_ids': [],
            'vpc_id': '123'
            }
"""

import json
import re

try:
    import botocore
except ImportError:
    pass  # caught by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry


@AWSRetry.jittered_backoff()
def _paginate(client, function, **params):
    paginator = client.get_paginator(function)
    return paginator.paginate(**params).build_full_result()


def alias_details(client, module, function_name):
    """
    Returns list of aliases for a specified function.

    :param client: AWS API client reference (boto3)
    :param module: Ansible module reference
    :param function_name (str): Name of Lambda function to query
    :return dict:
    """

    lambda_info = dict()

    try:
        lambda_info.update(aliases=_paginate(client, "list_aliases", FunctionName=function_name)["Aliases"])
    except is_boto3_error_code("ResourceNotFoundException"):
        lambda_info.update(aliases=[])
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Trying to get aliases")

    return camel_dict_to_snake_dict(lambda_info)


def _get_query(query, function_name):
    # create default values for query if not specified.
    # if function name exists, query should default to 'all'.
    # if function name does not exist, query should default to 'config' to limit the runtime when listing all lambdas.
    if query:
        return query
    if function_name:
        return "all"
    return "config"


def list_functions(client, module):
    """
    Returns queried facts for a specified function (or all functions).

    :param client: AWS API client reference (boto3)
    :param module: Ansible module reference
    """

    function_name = module.params.get("function_name")
    if function_name:
        # Function name is specified - retrieve info on that function
        function_names = [function_name]

    else:
        # Function name is not specified - retrieve all function names
        all_function_info = _paginate(client, "list_functions")["Functions"]
        function_names = [function_info["FunctionName"] for function_info in all_function_info]

    query = _get_query(module.params["query"], function_name)
    functions = []

    for function_name in function_names:
        function = {}

        # query = 'config' returns info such as FunctionName, FunctionArn, Description, etc
        # these details should be returned regardless of the query
        function.update(config_details(client, module, function_name))

        if query in ["all", "aliases"]:
            function.update(alias_details(client, module, function_name))

        if query in ["all", "policy"]:
            function.update(policy_details(client, module, function_name))

        if query in ["all", "versions"]:
            function.update(version_details(client, module, function_name))

        if query in ["all", "mappings"]:
            function.update(mapping_details(client, module, function_name))

        if query in ["all", "tags"]:
            function.update(tags_details(client, module, function_name))

        functions.append(function)

    module.exit_json(functions=functions, changed=False)


def config_details(client, module, function_name):
    """
    Returns configuration details for a lambda function.

    :param client: AWS API client reference (boto3)
    :param module: Ansible module reference
    :param function_name (str): Name of Lambda function to query
    :return dict:
    """

    lambda_info = dict()

    try:
        lambda_info.update(client.get_function_configuration(aws_retry=True, FunctionName=function_name))
    except is_boto3_error_code("ResourceNotFoundException"):
        lambda_info.update(function={})
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg=f"Trying to get {function_name} configuration")

    if "Environment" in lambda_info and "Variables" in lambda_info["Environment"]:
        env_vars = lambda_info["Environment"]["Variables"]
        snaked_lambda_info = camel_dict_to_snake_dict(lambda_info)
        snaked_lambda_info["environment"]["variables"] = env_vars
    else:
        snaked_lambda_info = camel_dict_to_snake_dict(lambda_info)

    return snaked_lambda_info


def mapping_details(client, module, function_name):
    """
    Returns all lambda event source mappings.

    :param client: AWS API client reference (boto3)
    :param module: Ansible module reference
    :param function_name (str): Name of Lambda function to query
    :return dict:
    """

    lambda_info = dict()
    params = dict()

    params["FunctionName"] = function_name

    if module.params.get("event_source_arn"):
        params["EventSourceArn"] = module.params.get("event_source_arn")

    try:
        lambda_info.update(mappings=_paginate(client, "list_event_source_mappings", **params)["EventSourceMappings"])
    except is_boto3_error_code("ResourceNotFoundException"):
        lambda_info.update(mappings=[])
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg="Trying to get source event mappings")

    return camel_dict_to_snake_dict(lambda_info)


def policy_details(client, module, function_name):
    """
    Returns policy attached to a lambda function.

    :param client: AWS API client reference (boto3)
    :param module: Ansible module reference
    :param function_name (str): Name of Lambda function to query
    :return dict:
    """

    lambda_info = dict()

    try:
        # get_policy returns a JSON string so must convert to dict before reassigning to its key
        lambda_info.update(policy=json.loads(client.get_policy(aws_retry=True, FunctionName=function_name)["Policy"]))
    except is_boto3_error_code("ResourceNotFoundException"):
        lambda_info.update(policy={})
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg=f"Trying to get {function_name} policy")

    return camel_dict_to_snake_dict(lambda_info)


def version_details(client, module, function_name):
    """
    Returns all lambda function versions.

    :param client: AWS API client reference (boto3)
    :param module: Ansible module reference
    :param function_name (str): Name of Lambda function to query
    :return dict:
    """

    lambda_info = dict()

    try:
        lambda_info.update(
            versions=_paginate(client, "list_versions_by_function", FunctionName=function_name)["Versions"]
        )
    except is_boto3_error_code("ResourceNotFoundException"):
        lambda_info.update(versions=[])
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg=f"Trying to get {function_name} versions")

    return camel_dict_to_snake_dict(lambda_info)


def tags_details(client, module, function_name):
    """
    Returns tag details for a lambda function.

    :param client: AWS API client reference (boto3)
    :param module: Ansible module reference
    :param function_name (str): Name of Lambda function to query
    :return dict:
    """

    lambda_info = dict()

    try:
        lambda_info.update(tags=client.get_function(aws_retry=True, FunctionName=function_name).get("Tags", {}))
    except is_boto3_error_code("ResourceNotFoundException"):
        lambda_info.update(function={})
    except (
        botocore.exceptions.ClientError,
        botocore.exceptions.BotoCoreError,
    ) as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e, msg=f"Trying to get {function_name} tags")

    return camel_dict_to_snake_dict(lambda_info)


def main():
    """
    Main entry point.

    :return dict: ansible facts
    """
    argument_spec = dict(
        function_name=dict(required=False, default=None, aliases=["function", "name"]),
        query=dict(
            required=False, choices=["aliases", "all", "config", "mappings", "policy", "versions", "tags"], default=None
        ),
        event_source_arn=dict(required=False, default=None),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec, supports_check_mode=True, mutually_exclusive=[], required_together=[]
    )

    # validate function_name if present
    function_name = module.params["function_name"]
    if function_name:
        if not re.search(r"^[\w\-:]+$", function_name):
            module.fail_json(
                msg=f"Function name {function_name} is invalid. Names must contain only alphanumeric characters and hyphens.",
            )
        if len(function_name) > 64:
            module.fail_json(msg=f'Function name "{function_name}" exceeds 64 character limit')

    client = module.client("lambda", retry_decorator=AWSRetry.jittered_backoff())

    list_functions(client, module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Akini Ross (@akinross) <akinross@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_dhcp_option_policy
short_description: Manage DHCP Option Policies on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage DHCP Option Policies on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Akini Ross (@akinross)
options:
  template:
    description:
    - The name of the template.
    - The template must be a tenant template.
    type: str
    required: true
  option_policy:
    description:
    - The name of the DHCP Option Policy.
    type: str
    aliases: [ name ]
  option_policy_uuid:
    description:
    - The uuid of the DHCP Option Policy.
    - This parameter is required when the O(option_policy) needs to be updated.
    type: str
    aliases: [ uuid ]
  description:
    description:
    - The description of the DHCP Option Policy.
    type: str
  options:
    description:
    - A list of options attached to the DHCP Option Policy.
    - The list of configured options must contain at least one option.
    - When the list of options is null the update will not change existing option configuration.
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - The name of the option.
        type: str
        required: true
      id:
        description:
        - The id of the option.
        type: int
      data:
        description:
        - The data of the option.
        type: str
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a new dhcp option policy
  cisco.mso.ndo_dhcp_option_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    option_policy: ansible_test_option_policy
    options:
      - name: option_1
        id: 1
        data: data_1
    state: present
  register: create

- name: Query a dhcp option policy with name
  cisco.mso.ndo_dhcp_option_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    option_policy: ansible_test_option_policy
    state: query
  register: query_one

- name: Query a dhcp option policy with UUID
  cisco.mso.ndo_dhcp_option_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    option_policy_uuid: '{{ create.current.uuid }}'
    state: query
  register: query_uuid

- name: Query all dhcp option policy in the template
  cisco.mso.ndo_dhcp_option_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    state: query
  register: query_all

- name: Delete a dhcp option policy
  cisco.mso.ndo_dhcp_option_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_tenant_template
    option_policy: ansible_test_option_policy
    state: absent
"""

RETURN = r"""
"""


import copy
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate, KVPair


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        template=dict(type="str", required=True),
        option_policy=dict(type="str", aliases=["name"]),
        option_policy_uuid=dict(type="str", aliases=["uuid"]),
        description=dict(type="str"),
        options=dict(
            type="list",
            elements="dict",
            options=dict(
                name=dict(type="str", required=True),
                id=dict(type="int"),
                data=dict(type="str"),
            ),
        ),
        state=dict(type="str", default="query", choices=["absent", "query", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["option_policy"]],
            ["state", "present", ["option_policy"]],
        ],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    option_policy = module.params.get("option_policy")
    option_policy_uuid = module.params.get("option_policy_uuid")
    options = get_options_payload(module.params.get("options")) if module.params.get("options") else []
    description = module.params.get("description")
    state = module.params.get("state")

    ops = []
    match = None
    err_message_min_options = "At least one option is required when state is present."

    mso_template = MSOTemplate(mso, "tenant", template)
    mso_template.validate_template("tenantPolicy")

    path = "/tenantPolicyTemplate/template/dhcpOptionPolicies"

    match = get_dhcp_option_policy(mso_template, option_policy_uuid, option_policy)

    if option_policy_uuid or option_policy:
        if match:
            mso.existing = mso.previous = copy.deepcopy(match.details)  # Query a specific object
    elif match:
        mso.existing = match  # Query all objects

    if state == "present":
        if match:
            if module.params.get("options") is not None and len(options) == 0:
                mso.fail_json(msg=err_message_min_options)

            if option_policy and match.details.get("name") != option_policy:
                ops.append(dict(op="replace", path="{0}/{1}/name".format(path, match.index), value=option_policy))
                match.details["name"] = option_policy

            if description is not None and match.details.get("description") != description:
                ops.append(dict(op="replace", path="{0}/{1}/description".format(path, match.index), value=description))
                match.details["description"] = description

            if module.params.get("options") is not None and match.details.get("options") != options:
                ops.append(dict(op="replace", path="{0}/{1}/options".format(path, match.index), value=options))
                match.details["options"] = options

            mso.sanitize(match.details)

        else:
            if not options:
                mso.fail_json(msg=err_message_min_options)

            payload = {"name": option_policy, "options": options}
            if description:
                payload["description"] = description

            ops.append(dict(op="add", path="{0}/-".format(path), value=payload))

            mso.sanitize(payload)

        mso.existing = mso.proposed

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path="{0}/{1}".format(path, match.index)))
        mso.existing = {}

    if not module.check_mode and ops:
        mso_template.template = mso.request(mso_template.template_path, method="PATCH", data=ops)
        match = get_dhcp_option_policy(mso_template, option_policy_uuid, option_policy)
        if match:
            mso.existing = match.details  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


def get_dhcp_option_policy(mso_template, uuid=None, name=None, fail_module=False):
    existing_dhcp_option_policies = mso_template.template.get("tenantPolicyTemplate", {}).get("template", {}).get("dhcpOptionPolicies", [])
    if uuid or name:  # Query a specific object
        return mso_template.get_object_by_key_value_pairs(
            "DHCP Option Policy", existing_dhcp_option_policies, [KVPair("uuid", uuid) if uuid else KVPair("name", name)], fail_module
        )
    return existing_dhcp_option_policies  # Query all objects


def get_options_payload(options):
    payload = []
    for option in options:
        option_payload = {"name": option.get("name")}
        if option.get("id") and id != 0:
            option_payload["id"] = option.get("id")
        if option.get("data"):
            option_payload["data"] = option.get("data")
        payload.append(option_payload)
    return payload


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the resource
#   module builder playbook.
#
# Do not edit this file manually.
#
# Changes to this file will be over written
#   by the resource module builder.
#
# Changes should be made in the model used to
#   generate this file or in the resource module
#   builder template.
#
#############################################

"""
The module file for nxos_acl_interfaces
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_acl_interfaces
short_description: ACL interfaces resource module
description: Add and remove Access Control Lists on interfaces in NX-OS platform
version_added: 1.0.0
author: Adharsh Srivats Rangarajan (@adharshsrivatsr)
notes:
- Tested against NX-OS 7.3.(0)D1(1) on VIRL
- Unsupported for Cisco MDS
options:
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the NX-OS device
      by executing the command B(show running-config | section '^interface').
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  config:
    description: A list of interfaces to be configured with ACLs
    type: list
    elements: dict
    suboptions:
      name:
        description: Name of the interface
        type: str
        required: true
      access_groups:
        description: List of address family indicators with ACLs to be configured
          on the interface
        type: list
        elements: dict
        suboptions:
          afi:
            description: Address Family Indicator of the ACLs to be configured
            type: str
            required: true
            choices:
            - ipv4
            - ipv6
          acls:
            description: List of Access Control Lists for the interface
            type: list
            elements: dict
            suboptions:
              name:
                description: Name of the ACL to be added/removed
                type: str
                required: true
              direction:
                description: Direction to be applied for the ACL
                type: str
                required: true
                choices:
                - in
                - out
              port:
                description: Use ACL as port policy.
                type: bool
  state:
    description: The state the configuration should be left in
    type: str
    choices:
    - deleted
    - gathered
    - merged
    - overridden
    - rendered
    - replaced
    - parsed
    default: merged

"""
EXAMPLES = """
# Using merged

# Before state:
# ------------
#

- name: Merge ACL interfaces configuration
  cisco.nxos.nxos_acl_interfaces:
    config:
      - name: Ethernet1/2
        access_groups:
          - afi: ipv6
            acls:
              - name: ACL1v6
                direction: in

      - name: Eth1/5
        access_groups:
          - afi: ipv4
            acls:
              - name: PortACL
                direction: in
                port: true

              - name: ACL1v4
                direction: out

          - afi: ipv6
            acls:
              - name: ACL1v6
                direction: in
    state: merged

# After state:
# ------------
# interface Ethernet1/2
#   ipv6 traffic-filter ACL1v6 in
# interface Ethernet1/5
#   ip port access-group PortACL in
#   ip access-group ACL1v4 out
#   ipv6 traffic-filter ACL1v6 in

# Using replaced

# Before state:
# ------------
# interface Ethernet1/2
#   ipv6 traffic-filter ACL1v6 in
# interface Ethernet1/5
#   ip port access-group PortACL in
#   ip access-group ACL1v4 out
#   ipv6 traffic-filter ACL1v6 in

- name: Replace interface configuration with given configuration
  cisco.nxos.nxos_acl_interfaces:
    config:
      - name: Eth1/5
        access_groups:
          - afi: ipv4
            acls:
              - name: NewACLv4
                direction: out

      - name: Ethernet1/3
        access_groups:
          - afi: ipv6
            acls:
              - name: NewACLv6
                direction: in
                port: true
    state: replaced

# After state:
# ------------
# interface Ethernet1/2
#   ipv6 traffic-filter ACL1v6 in
# interface Ethernet1/3
#   ipv6 port traffic-filter NewACLv6 in
# interface Ethernet1/5
#   ip access-group NewACLv4 out

# Using overridden

# Before state:
# ------------
# interface Ethernet1/2
#   ipv6 traffic-filter ACL1v6 in
# interface Ethernet1/5
#   ip port access-group PortACL in
#   ip access-group ACL1v4 out
#   ipv6 traffic-filter ACL1v6 in

- name: Override interface configuration with given configuration
  cisco.nxos.nxos_acl_interfaces:
    config:
      - name: Ethernet1/3
        access_groups:
          - afi: ipv4
            acls:
              - name: ACL1v4
                direction: out

              - name: PortACL
                port: true
                direction: in
          - afi: ipv6
            acls:
              - name: NewACLv6
                direction: in
                port: true
    state: overridden

# After state:
# ------------
# interface Ethernet1/3
#   ip access-group ACL1v4 out
#   ip port access-group PortACL in
#   ipv6 port traffic-filter NewACLv6 in

# Using deleted to remove ACL config from specified interfaces

# Before state:
# -------------
# interface Ethernet1/1
#   ip access-group ACL2v4 in
# interface Ethernet1/2
#   ipv6 traffic-filter ACL1v6 in
# interface Ethernet1/5
#   ip port access-group PortACL in
#   ip access-group ACL1v4 out
#   ipv6 traffic-filter ACL1v6 in

- name: Delete ACL configuration on interfaces
  cisco.nxos.nxos_acl_interfaces:
    config:
      - name: Ethernet1/5
      - name: Ethernet1/2
    state: deleted

# After state:
# -------------
# interface Ethernet1/1
#   ip access-group ACL2v4 in
# interface Ethernet1/2
# interface Ethernet1/5

# Using deleted to remove ACL config from all interfaces

# Before state:
# -------------
# interface Ethernet1/1
#   ip access-group ACL2v4 in
# interface Ethernet1/2
#   ipv6 traffic-filter ACL1v6 in
# interface Ethernet1/5
#   ip port access-group PortACL in
#   ip access-group ACL1v4 out
#   ipv6 traffic-filter ACL1v6 in

- name: Delete ACL configuration from all interfaces
  cisco.nxos.nxos_acl_interfaces:
    state: deleted

# After state:
# -------------
# interface Ethernet1/1
# interface Ethernet1/2
# interface Ethernet1/5

# Using parsed

- name: Parse given configuration into structured format
  cisco.nxos.nxos_acl_interfaces:
    running_config: |
      interface Ethernet1/2
      ipv6 traffic-filter ACL1v6 in
      interface Ethernet1/5
      ipv6 traffic-filter ACL1v6 in
      ip access-group ACL1v4 out
      ip port access-group PortACL in
    state: parsed

# returns
# parsed:
#   - name: Ethernet1/2
#     access_groups:
#       - afi: ipv6
#         acls:
#           - name: ACL1v6
#             direction: in
#  - name: Ethernet1/5
#    access_groups:
#      - afi: ipv4
#        acls:
#          - name: PortACL
#            direction: in
#            port: True
#          - name: ACL1v4
#            direction: out
#      - afi: ipv6
#        acls:
#          - name: ACL1v6
#             direction: in


# Using gathered:

# Before state:
# ------------
# interface Ethernet1/2
#   ipv6 traffic-filter ACL1v6 in
# interface Ethernet1/5
#   ipv6 traffic-filter ACL1v6 in
#   ip access-group ACL1v4 out
#   ip port access-group PortACL in

- name: Gather existing configuration from device
  cisco.nxos.nxos_acl_interfaces:
    config:
    state: gathered

# returns
# gathered:
#   - name: Ethernet1/2
#     access_groups:
#       - afi: ipv6
#         acls:
#           - name: ACL1v6
#             direction: in
#  - name: Ethernet1/5
#    access_groups:
#      - afi: ipv4
#        acls:
#          - name: PortACL
#            direction: in
#            port: True
#          - name: ACL1v4
#            direction: out
#      - afi: ipv6
#        acls:
#          - name: ACL1v6
#             direction: in


# Using rendered

- name: Render required configuration to be pushed to the device
  cisco.nxos.nxos_acl_interfaces:
    config:
      - name: Ethernet1/2
        access_groups:
          - afi: ipv6
            acls:
              - name: ACL1v6
                direction: in
      - name: Ethernet1/5
        access_groups:
          - afi: ipv4
            acls:
              - name: PortACL
                direction: in
                port: true
              - name: ACL1v4
                direction: out
          - afi: ipv6
            acls:
              - name: ACL1v6
                direction: in
    state: rendered


# returns
# rendered:
#   interface Ethernet1/2
#   ipv6 traffic-filter ACL1v6 in
#   interface Ethernet1/5
#   ipv6 traffic-filter ACL1v6 in
#   ip access-group ACL1v4 out
#   ip port access-group PortACL in
"""
RETURN = """
before:
  description: The configuration prior to the model invocation.
  returned: always
  type: dict
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
after:
  description: The resulting configuration model invocation.
  returned: when changed
  type: dict
  sample: >
    The configuration returned will always be in the same format
     of the parameters above.
commands:
  description: The set of commands pushed to the remote device.
  returned: always
  type: list
  sample: ['interface Ethernet1/2', 'ipv6 traffic-filter ACL1v6 out', 'ip port access-group PortACL in']
"""


from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.acl_interfaces.acl_interfaces import (
    Acl_interfacesArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.config.acl_interfaces.acl_interfaces import (
    Acl_interfaces,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Acl_interfacesArgs.argument_spec,
        supports_check_mode=True,
    )

    result = Acl_interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2017, Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ipa_hbacrule
author: Thomas Krahn (@Nosmoht)
short_description: Manage FreeIPA HBAC rule
description:
  - Add, modify or delete an IPA HBAC rule using IPA API.
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  cn:
    description:
      - Canonical name.
      - Can not be changed as it is the unique identifier.
    required: true
    aliases: ["name"]
    type: str
  description:
    description: Description.
    type: str
  host:
    description:
      - List of host names to assign.
      - If an empty list is passed all hosts are removed from the rule.
      - If option is omitted hosts are not checked or changed.
    type: list
    elements: str
  hostcategory:
    description: Host category.
    choices: ['all']
    type: str
  hostgroup:
    description:
      - List of hostgroup names to assign.
      - If an empty list is passed all hostgroups are removed from the rule.
      - If option is omitted hostgroups are not checked or changed.
    type: list
    elements: str
  service:
    description:
      - List of service names to assign.
      - If an empty list is passed all services are removed from the rule.
      - If option is omitted services are not checked or changed.
    type: list
    elements: str
  servicecategory:
    description: Service category.
    choices: ['all']
    type: str
  servicegroup:
    description:
      - List of service group names to assign.
      - If an empty list is passed all assigned service groups are removed from the rule.
      - If option is omitted service groups are not checked or changed.
    type: list
    elements: str
  sourcehost:
    description:
      - List of source host names to assign.
      - If an empty list if passed all assigned source hosts are removed from the rule.
      - If option is omitted source hosts are not checked or changed.
    type: list
    elements: str
  sourcehostcategory:
    description: Source host category.
    choices: ['all']
    type: str
  sourcehostgroup:
    description:
      - List of source host group names to assign.
      - If an empty list if passed all assigned source host groups are removed from the rule.
      - If option is omitted source host groups are not checked or changed.
    type: list
    elements: str
  state:
    description: State to ensure.
    default: "present"
    choices: ["absent", "disabled", "enabled", "present"]
    type: str
  user:
    description:
      - List of user names to assign.
      - If an empty list if passed all assigned users are removed from the rule.
      - If option is omitted users are not checked or changed.
    type: list
    elements: str
  usercategory:
    description: User category.
    choices: ['all']
    type: str
  usergroup:
    description:
      - List of user group names to assign.
      - If an empty list if passed all assigned user groups are removed from the rule.
      - If option is omitted user groups are not checked or changed.
    type: list
    elements: str
extends_documentation_fragment:
  - community.general._ipa.documentation
  - community.general._ipa.connection_notes
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Ensure rule to allow all users to access any host from any host
  community.general.ipa_hbacrule:
    name: allow_all
    description: Allow all users to access any host from any host
    hostcategory: all
    servicecategory: all
    usercategory: all
    state: present
    ipa_host: ipa.example.com
    ipa_user: admin
    ipa_pass: topsecret

- name: Ensure rule with certain limitations
  community.general.ipa_hbacrule:
    name: allow_all_developers_access_to_db
    description: Allow all developers to access any database from any host
    hostgroup:
      - db-server
    usergroup:
      - developers
    state: present
    ipa_host: ipa.example.com
    ipa_user: admin
    ipa_pass: topsecret

- name: Ensure rule is absent
  community.general.ipa_hbacrule:
    name: rule_to_be_deleted
    state: absent
    ipa_host: ipa.example.com
    ipa_user: admin
    ipa_pass: topsecret
"""

RETURN = r"""
hbacrule:
  description: HBAC rule as returned by IPA API.
  returned: always
  type: dict
"""

import traceback

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._ipa import IPAClient, ipa_argument_spec
from ansible_collections.community.general.plugins.module_utils._version import LooseVersion


class HBACRuleIPAClient(IPAClient):
    def __init__(self, module, host, port, protocol):
        super().__init__(module, host, port, protocol)

    def hbacrule_find(self, name):
        return self._post_json(method="hbacrule_find", name=None, item={"all": True, "cn": name})

    def hbacrule_add(self, name, item):
        return self._post_json(method="hbacrule_add", name=name, item=item)

    def hbacrule_mod(self, name, item):
        return self._post_json(method="hbacrule_mod", name=name, item=item)

    def hbacrule_del(self, name):
        return self._post_json(method="hbacrule_del", name=name)

    def hbacrule_add_host(self, name, item):
        return self._post_json(method="hbacrule_add_host", name=name, item=item)

    def hbacrule_remove_host(self, name, item):
        return self._post_json(method="hbacrule_remove_host", name=name, item=item)

    def hbacrule_add_service(self, name, item):
        return self._post_json(method="hbacrule_add_service", name=name, item=item)

    def hbacrule_remove_service(self, name, item):
        return self._post_json(method="hbacrule_remove_service", name=name, item=item)

    def hbacrule_add_user(self, name, item):
        return self._post_json(method="hbacrule_add_user", name=name, item=item)

    def hbacrule_remove_user(self, name, item):
        return self._post_json(method="hbacrule_remove_user", name=name, item=item)

    def hbacrule_add_sourcehost(self, name, item):
        return self._post_json(method="hbacrule_add_sourcehost", name=name, item=item)

    def hbacrule_remove_sourcehost(self, name, item):
        return self._post_json(method="hbacrule_remove_sourcehost", name=name, item=item)


def get_hbacrule_dict(
    description=None,
    hostcategory=None,
    ipaenabledflag=None,
    servicecategory=None,
    sourcehostcategory=None,
    usercategory=None,
):
    data = {}
    if description is not None:
        data["description"] = description
    if hostcategory is not None:
        data["hostcategory"] = hostcategory
    if ipaenabledflag is not None:
        data["ipaenabledflag"] = ipaenabledflag
    if servicecategory is not None:
        data["servicecategory"] = servicecategory
    if sourcehostcategory is not None:
        data["sourcehostcategory"] = sourcehostcategory
    if usercategory is not None:
        data["usercategory"] = usercategory
    return data


def get_hbcarule_diff(client, ipa_hbcarule, module_hbcarule):
    return client.get_diff(ipa_data=ipa_hbcarule, module_data=module_hbcarule)


def ensure(module, client):
    name = module.params["cn"]
    state = module.params["state"]

    ipa_version = client.get_ipa_version()
    if state in ["present", "enabled"]:
        if LooseVersion(ipa_version) < LooseVersion("4.9.10"):
            ipaenabledflag = "TRUE"
        else:
            ipaenabledflag = True
    else:
        if LooseVersion(ipa_version) < LooseVersion("4.9.10"):
            ipaenabledflag = "FALSE"
        else:
            ipaenabledflag = False

    host = module.params["host"]
    hostcategory = module.params["hostcategory"]
    hostgroup = module.params["hostgroup"]
    service = module.params["service"]
    servicecategory = module.params["servicecategory"]
    servicegroup = module.params["servicegroup"]
    sourcehost = module.params["sourcehost"]
    sourcehostcategory = module.params["sourcehostcategory"]
    sourcehostgroup = module.params["sourcehostgroup"]
    user = module.params["user"]
    usercategory = module.params["usercategory"]
    usergroup = module.params["usergroup"]

    module_hbacrule = get_hbacrule_dict(
        description=module.params["description"],
        hostcategory=hostcategory,
        ipaenabledflag=ipaenabledflag,
        servicecategory=servicecategory,
        sourcehostcategory=sourcehostcategory,
        usercategory=usercategory,
    )
    ipa_hbacrule = client.hbacrule_find(name=name)

    changed = False
    if state in ["present", "enabled", "disabled"]:
        if not ipa_hbacrule:
            changed = True
            if not module.check_mode:
                ipa_hbacrule = client.hbacrule_add(name=name, item=module_hbacrule)
        else:
            diff = get_hbcarule_diff(client, ipa_hbacrule, module_hbacrule)
            if len(diff) > 0:
                changed = True
                if not module.check_mode:
                    data = {}
                    for key in diff:
                        data[key] = module_hbacrule.get(key)
                    client.hbacrule_mod(name=name, item=data)

        if host is not None:
            changed = (
                client.modify_if_diff(
                    name,
                    ipa_hbacrule.get("memberhost_host", []),
                    host,
                    client.hbacrule_add_host,
                    client.hbacrule_remove_host,
                    "host",
                )
                or changed
            )

        if hostgroup is not None:
            changed = (
                client.modify_if_diff(
                    name,
                    ipa_hbacrule.get("memberhost_hostgroup", []),
                    hostgroup,
                    client.hbacrule_add_host,
                    client.hbacrule_remove_host,
                    "hostgroup",
                )
                or changed
            )

        if service is not None:
            changed = (
                client.modify_if_diff(
                    name,
                    ipa_hbacrule.get("memberservice_hbacsvc", []),
                    service,
                    client.hbacrule_add_service,
                    client.hbacrule_remove_service,
                    "hbacsvc",
                )
                or changed
            )

        if servicegroup is not None:
            changed = (
                client.modify_if_diff(
                    name,
                    ipa_hbacrule.get("memberservice_hbacsvcgroup", []),
                    servicegroup,
                    client.hbacrule_add_service,
                    client.hbacrule_remove_service,
                    "hbacsvcgroup",
                )
                or changed
            )

        if sourcehost is not None:
            changed = (
                client.modify_if_diff(
                    name,
                    ipa_hbacrule.get("sourcehost_host", []),
                    sourcehost,
                    client.hbacrule_add_sourcehost,
                    client.hbacrule_remove_sourcehost,
                    "host",
                )
                or changed
            )

        if sourcehostgroup is not None:
            changed = (
                client.modify_if_diff(
                    name,
                    ipa_hbacrule.get("sourcehost_group", []),
                    sourcehostgroup,
                    client.hbacrule_add_sourcehost,
                    client.hbacrule_remove_sourcehost,
                    "hostgroup",
                )
                or changed
            )

        if user is not None:
            changed = (
                client.modify_if_diff(
                    name,
                    ipa_hbacrule.get("memberuser_user", []),
                    user,
                    client.hbacrule_add_user,
                    client.hbacrule_remove_user,
                    "user",
                )
                or changed
            )

        if usergroup is not None:
            changed = (
                client.modify_if_diff(
                    name,
                    ipa_hbacrule.get("memberuser_group", []),
                    usergroup,
                    client.hbacrule_add_user,
                    client.hbacrule_remove_user,
                    "group",
                )
                or changed
            )
    else:
        if ipa_hbacrule:
            changed = True
            if not module.check_mode:
                client.hbacrule_del(name=name)

    return changed, client.hbacrule_find(name=name)


def main():
    argument_spec = ipa_argument_spec()
    argument_spec.update(
        cn=dict(type="str", required=True, aliases=["name"]),
        description=dict(type="str"),
        host=dict(type="list", elements="str"),
        hostcategory=dict(type="str", choices=["all"]),
        hostgroup=dict(type="list", elements="str"),
        service=dict(type="list", elements="str"),
        servicecategory=dict(type="str", choices=["all"]),
        servicegroup=dict(type="list", elements="str"),
        sourcehost=dict(type="list", elements="str"),
        sourcehostcategory=dict(type="str", choices=["all"]),
        sourcehostgroup=dict(type="list", elements="str"),
        state=dict(type="str", default="present", choices=["present", "absent", "enabled", "disabled"]),
        user=dict(type="list", elements="str"),
        usercategory=dict(type="str", choices=["all"]),
        usergroup=dict(type="list", elements="str"),
    )

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    client = HBACRuleIPAClient(
        module=module,
        host=module.params["ipa_host"],
        port=module.params["ipa_port"],
        protocol=module.params["ipa_prot"],
    )

    try:
        client.login(username=module.params["ipa_user"], password=module.params["ipa_pass"])
        changed, hbacrule = ensure(module, client)
        module.exit_json(changed=changed, hbacrule=hbacrule)
    except Exception as e:
        module.fail_json(msg=f"{e}", exception=traceback.format_exc())


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_summary_top_networks_by_status_info
short_description: Information module for organizations _summary _top _networks _by
  _status
description:
  - Get all organizations _summary _top _networks _by _status. - > List the client
    and status overview information for the networks in an organization. Usage is
    measured in kilobytes and from the last seven days.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - OrganizationId path parameter. Organization ID.
    type: str
  networkTag:
    description:
      - NetworkTag query parameter. Match result to an exact network tag.
    type: str
  deviceTag:
    description:
      - DeviceTag query parameter. Match result to an exact device tag.
    type: str
  quantity:
    description:
      - Quantity query parameter. Set number of desired results to return. Default
        is 10. Maximum is 50.
    type: int
  ssidName:
    description:
      - SsidName query parameter. Filter results by ssid name.
    type: str
  usageUplink:
    description:
      - UsageUplink query parameter. Filter results by usage uplink.
    type: str
  perPage:
    description:
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 5000.
    type: int
  startingAfter:
    description:
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationSummaryTopNetworksByStatus
    description: Complete reference of the getOrganizationSummaryTopNetworksByStatus
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-summary-top-networks-by-status
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_summary_top_networks_by_status,
  - Paths used are
    get /organizations/{organizationId}/summary/top/networks/byStatus,
"""

EXAMPLES = r"""
- name: Get all organizations _summary _top _networks _by _status
  cisco.meraki.organizations_summary_top_networks_by_status_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    networkTag: string
    deviceTag: string
    quantity: 0
    ssidName: string
    usageUplink: string
    perPage: 0
    startingAfter: string
    endingBefore: string
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "networkId": "string",
        "name": "string",
        "url": "string",
        "tags": [
          "string"
        ],
        "group": {
          "id": "string"
        },
        "clients": {
          "counts": {
            "total": 0
          },
          "usage": {
            "upstream": 0,
            "downstream": 0
          }
        },
        "statuses": {
          "overall": "string",
          "byProductType": [
            {
              "productType": "string",
              "counts": {
                "online": 0,
                "offline": 0,
                "alerting": 0,
                "dormant": 0
              }
            }
          ]
        },
        "devices": {
          "byProductType": [
            {
              "productType": "string",
              "url": "string"
            }
          ]
        },
        "productTypes": [
          "string"
        ]
      }
    ]
"""



#!/usr/bin/python

# Copyright (c) 2013-2014, Christian Berendt <berendt@b1-systems.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: apache2_module
author:
  - Christian Berendt (@berendt)
  - Ralf Hertel (@n0trax)
  - Robin Roth (@robinro)
short_description: Enables/disables a module of the Apache2 webserver
description:
  - Enables or disables a specified module of the Apache2 webserver.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  name:
    type: str
    description:
      - Name of the module to enable/disable as given to C(a2enmod)/C(a2dismod).
    required: true
  identifier:
    type: str
    description:
      - Identifier of the module as listed by C(apache2ctl -M). This is optional and usually determined automatically by the
        common convention of appending V(_module) to O(name) as well as custom exception for popular modules.
  force:
    description:
      - Force disabling of default modules and override Debian warnings.
    type: bool
    default: false
  state:
    type: str
    description:
      - Desired state of the module.
    choices: ['present', 'absent']
    default: present
  ignore_configcheck:
    description:
      - Ignore configuration checks about inconsistent module configuration. Especially for mpm_* modules.
    type: bool
    default: false
  warn_mpm_absent:
    description:
      - Control the behavior of the warning process for MPM modules.
    type: bool
    default: true
    version_added: 6.3.0
requirements: ["a2enmod", "a2dismod"]
notes:
  - This does not work on RedHat-based distributions. It does work on Debian- and SuSE-based distributions. Whether it works
    on others depend on whether the C(a2enmod) and C(a2dismod) tools are available or not.
"""

EXAMPLES = r"""
- name: Enable the Apache2 module wsgi
  community.general.apache2_module:
    state: present
    name: wsgi

- name: Disables the Apache2 module wsgi
  community.general.apache2_module:
    state: absent
    name: wsgi

- name: Disable default modules for Debian
  community.general.apache2_module:
    state: absent
    name: autoindex
    force: true

- name: Disable mpm_worker and ignore warnings about missing mpm module
  community.general.apache2_module:
    state: absent
    name: mpm_worker
    ignore_configcheck: true

- name: Disable mpm_event, enable mpm_prefork and ignore warnings about missing mpm module
  community.general.apache2_module:
    name: "{{ item.module }}"
    state: "{{ item.state }}"
    warn_mpm_absent: false
    ignore_configcheck: true
  loop:
    - module: mpm_event
      state: absent
    - module: mpm_prefork
      state: present

- name: Enable dump_io module, which is identified as dumpio_module inside apache2
  community.general.apache2_module:
    state: present
    name: dump_io
    identifier: dumpio_module
"""

RETURN = r"""
result:
  description: Message about action taken.
  returned: always
  type: str
"""

import re

# import module snippets
from ansible.module_utils.basic import AnsibleModule

_re_threaded = re.compile(r"threaded: *yes")


def _run_threaded(module):
    control_binary = _get_ctl_binary(module)
    result, stdout, stderr = module.run_command([control_binary, "-V"])

    return bool(_re_threaded.search(stdout))


def _get_ctl_binary(module):
    for command in ["apache2ctl", "apachectl"]:
        ctl_binary = module.get_bin_path(command)
        if ctl_binary is not None:
            return ctl_binary

    module.fail_json(msg="Neither of apache2ctl nor apachectl found. At least one apache control binary is necessary.")


def _module_is_enabled(module):
    control_binary = _get_ctl_binary(module)
    result, stdout, stderr = module.run_command([control_binary, "-M"])

    if result != 0:
        error_msg = f"Error executing {control_binary}: {stderr}"
        if module.params["ignore_configcheck"]:
            if "AH00534" in stderr and "mpm_" in module.params["name"]:
                if module.params["warn_mpm_absent"]:
                    module.warn(
                        "No MPM module loaded! apache2 reload AND other module actions"
                        " will fail if no MPM module is loaded immediately."
                    )
            else:
                module.warn(error_msg)
            return False
        else:
            module.fail_json(msg=error_msg)

    searchstring = f" {module.params['identifier']}"
    return searchstring in stdout


def create_apache_identifier(name):
    """
    By convention if a module is loaded via name, it appears in apache2ctl -M as
    name_module.

    Some modules don't follow this convention and we use replacements for those."""

    # a2enmod name replacement to apache2ctl -M names
    text_workarounds = [
        ("shib", "mod_shib"),
        ("shib2", "mod_shib"),
        ("evasive", "evasive20_module"),
    ]

    # re expressions to extract subparts of names
    re_workarounds = [
        ("php8", re.compile(r"^(php)[\d\.]+")),
        ("php", re.compile(r"^(php\d)\.")),
    ]

    for a2enmod_spelling, module_name in text_workarounds:
        if a2enmod_spelling in name:
            return module_name

    for search, reexpr in re_workarounds:
        if search in name:
            try:
                rematch = reexpr.search(name)
                return f"{rematch.group(1)}_module"
            except AttributeError:
                pass

    return f"{name}_module"


def _set_state(module, state):
    name = module.params["name"]
    force = module.params["force"]

    want_enabled = state == "present"
    state_string = {"present": "enabled", "absent": "disabled"}[state]
    a2mod_binary = {"present": "a2enmod", "absent": "a2dismod"}[state]
    success_msg = f"Module {name} {state_string}"

    if _module_is_enabled(module) != want_enabled:
        if module.check_mode:
            module.exit_json(changed=True, result=success_msg)

        a2mod_binary_path = module.get_bin_path(a2mod_binary)
        if a2mod_binary_path is None:
            module.fail_json(
                msg=f"{a2mod_binary} not found. Perhaps this system does not use {a2mod_binary} to manage apache"
            )

        a2mod_binary_cmd = [a2mod_binary_path]

        if not want_enabled and force:
            # force exists only for a2dismod on debian
            a2mod_binary_cmd.append("-f")

        result, stdout, stderr = module.run_command(a2mod_binary_cmd + [name])

        if _module_is_enabled(module) == want_enabled:
            module.exit_json(changed=True, result=success_msg)
        else:
            msg = (
                f"Failed to set module {name} to {state_string}:\n"
                f"{stdout}\n"
                f"Maybe the module identifier ({module.params['identifier']}) was guessed incorrectly."
                'Consider setting the "identifier" option.'
            )
            module.fail_json(msg=msg, rc=result, stdout=stdout, stderr=stderr)
    else:
        module.exit_json(changed=False, result=success_msg)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(required=True),
            identifier=dict(type="str"),
            force=dict(type="bool", default=False),
            state=dict(default="present", choices=["absent", "present"]),
            ignore_configcheck=dict(type="bool", default=False),
            warn_mpm_absent=dict(type="bool", default=True),
        ),
        supports_check_mode=True,
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    name = module.params["name"]
    if name == "cgi" and module.params["state"] == "present" and _run_threaded(module):
        module.fail_json(msg="Your MPM seems to be threaded, therefore enabling cgi module is not allowed.")

    if not module.params["identifier"]:
        module.params["identifier"] = create_apache_identifier(module.params["name"])

    if module.params["state"] in ["present", "absent"]:
        _set_state(module, module.params["state"])


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2019 Piotr Wojciechowski <piotr@it-playground.pl>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: docker_host_info

short_description: Retrieves facts about docker host and lists of objects of the services

description:
  - Retrieves facts about a docker host.
  - Essentially returns the output of C(docker system info).
  - The module also allows to list object names for containers, images, networks and volumes. It also allows to query information
    on disk usage.
  - The output differs depending on API version of the docker daemon.
  - If the docker daemon cannot be contacted or does not meet the API version requirements, the module will fail.
extends_documentation_fragment:
  - community.docker._docker.api_documentation
  - community.docker._attributes
  - community.docker._attributes.actiongroup_docker
  - community.docker._attributes.idempotent_not_modify_state

attributes:
  check_mode:
    support: full
    details:
      - This action does not modify state.
  diff_mode:
    support: N/A
    details:
      - This action does not modify state.

options:
  containers:
    description:
      - Whether to list containers.
    type: bool
    default: false
  containers_all:
    description:
      - By default, only running containers are returned.
      - This corresponds to the C(--all) option to C(docker container list).
    type: bool
    default: false
    version_added: 3.4.0
  containers_filters:
    description:
      - A dictionary of filter values used for selecting containers to list.
      - 'For example, C(until: 24h).'
      - C(label) is a special case of filter which can be a string C(<key>) matching when a label is present, a string C(<key>=<value>)
        matching when a label has a particular value, or a list of strings C(<key>)/C(<key>=<value).
      - See L(the docker documentation,https://docs.docker.com/engine/reference/commandline/container_prune/#filtering) for
        more information on possible filters.
    type: dict
  images:
    description:
      - Whether to list images.
    type: bool
    default: false
  images_filters:
    description:
      - A dictionary of filter values used for selecting images to list.
      - 'For example, C(dangling: true).'
      - C(label) is a special case of filter which can be a string C(<key>) matching when a label is present, a string C(<key>=<value>)
        matching when a label has a particular value, or a list of strings C(<key>)/C(<key>=<value).
      - See L(the docker documentation,https://docs.docker.com/engine/reference/commandline/image_prune/#filtering) for more
        information on possible filters.
    type: dict
  networks:
    description:
      - Whether to list networks.
    type: bool
    default: false
  networks_filters:
    description:
      - A dictionary of filter values used for selecting networks to list.
      - C(label) is a special case of filter which can be a string C(<key>) matching when a label is present, a string C(<key>=<value>)
        matching when a label has a particular value, or a list of strings C(<key>)/C(<key>=<value).
      - See L(the docker documentation,https://docs.docker.com/engine/reference/commandline/network_prune/#filtering) for
        more information on possible filters.
    type: dict
  volumes:
    description:
      - Whether to list volumes.
    type: bool
    default: false
  volumes_filters:
    description:
      - A dictionary of filter values used for selecting volumes to list.
      - C(label) is a special case of filter which can be a string C(<key>) matching when a label is present, a string C(<key>=<value>)
        matching when a label has a particular value, or a list of strings C(<key>)/C(<key>=<value).
      - See L(the docker documentation,https://docs.docker.com/engine/reference/commandline/volume_prune/#filtering) for more
        information on possible filters.
    type: dict
  disk_usage:
    description:
      - Summary information on used disk space by all Docker layers.
      - The output is a sum of images, volumes, containers and build cache.
    type: bool
    default: false
  verbose_output:
    description:
      - When set to V(true) and O(networks), O(volumes), O(images), O(containers), or O(disk_usage) is set to V(true) then
        output will contain verbose information about objects matching the full output of API method. For details see the
        documentation of your version of Docker API at U(https://docs.docker.com/engine/api/).
      - The verbose output in this module contains only subset of information returned by this module for each type of the
        objects.
    type: bool
    default: false

author:
  - Piotr Wojciechowski (@WojciechowskiPiotr)

requirements:
  - "Docker API >= 1.25"
"""

EXAMPLES = r"""
---
- name: Get info on docker host
  community.docker.docker_host_info:
  register: result

- name: Get info on docker host and list images
  community.docker.docker_host_info:
    images: true
  register: result

- name: Get info on docker host and list images matching the filter
  community.docker.docker_host_info:
    images: true
    images_filters:
      label: "mylabel"
  register: result

- name: Get info on docker host and verbose list images
  community.docker.docker_host_info:
    images: true
    verbose_output: true
  register: result

- name: Get info on docker host and used disk space
  community.docker.docker_host_info:
    disk_usage: true
  register: result

- name: Get info on docker host and list containers matching the filter
  community.docker.docker_host_info:
    containers: true
    containers_filters:
      label:
        - key1=value1
        - key2=value2
  register: result

- name: Show host information
  ansible.builtin.debug:
    var: result.host_info
"""

RETURN = r"""
can_talk_to_docker:
  description:
    - Will be V(true) if the module can talk to the docker daemon.
  returned: both on success and on error
  type: bool

host_info:
  description:
    - Facts representing the basic state of the docker host. Matches the C(docker system info) output.
  returned: always
  type: dict
volumes:
  description:
    - List of dict objects containing the basic information about each volume. Keys matches the C(docker volume ls) output
      unless O(verbose_output=true). See description for O(verbose_output).
  returned: When O(volumes=true)
  type: list
  elements: dict
networks:
  description:
    - List of dict objects containing the basic information about each network. Keys matches the C(docker network ls) output
      unless O(verbose_output=true). See description for O(verbose_output).
  returned: When O(networks=true)
  type: list
  elements: dict
containers:
  description:
    - List of dict objects containing the basic information about each container. Keys matches the C(docker container ls)
      output unless O(verbose_output=true). See description for O(verbose_output).
  returned: When O(containers=true)
  type: list
  elements: dict
images:
  description:
    - List of dict objects containing the basic information about each image. Keys matches the C(docker image ls) output unless
      O(verbose_output=true). See description for O(verbose_output).
  returned: When O(images=true)
  type: list
  elements: dict
disk_usage:
  description:
    - Information on summary disk usage by images, containers and volumes on docker host unless O(verbose_output=true). See
      description for O(verbose_output).
  returned: When O(disk_usage=true)
  type: dict
"""

import traceback
import typing as t

from ansible_collections.community.docker.plugins.module_utils._api.errors import (
    APIError,
    DockerException,
)
from ansible_collections.community.docker.plugins.module_utils._api.utils.utils import (
    convert_filters,
)
from ansible_collections.community.docker.plugins.module_utils._common_api import (
    AnsibleDockerClient,
    RequestException,
)
from ansible_collections.community.docker.plugins.module_utils._util import (
    DockerBaseClass,
    clean_dict_booleans_for_docker_api,
)


class DockerHostManager(DockerBaseClass):
    def __init__(self, client: AnsibleDockerClient, results: dict[str, t.Any]) -> None:
        super().__init__()

        self.client = client
        self.results = results
        self.verbose_output = self.client.module.params["verbose_output"]

        listed_objects = ["volumes", "networks", "containers", "images"]

        self.results["host_info"] = self.get_docker_host_info()
        # At this point we definitely know that we can talk to the Docker daemon
        self.results["can_talk_to_docker"] = True
        self.client.fail_results["can_talk_to_docker"] = True

        if self.client.module.params["disk_usage"]:
            self.results["disk_usage"] = self.get_docker_disk_usage_facts()

        for docker_object in listed_objects:
            if self.client.module.params[docker_object]:
                returned_name = docker_object
                filter_name = f"{docker_object}_filters"
                filters = clean_dict_booleans_for_docker_api(
                    client.module.params.get(filter_name), allow_sequences=True
                )
                self.results[returned_name] = self.get_docker_items_list(
                    docker_object, filters
                )

    def get_docker_host_info(self) -> dict[str, t.Any]:
        try:
            return self.client.info()
        except APIError as exc:
            self.client.fail(f"Error inspecting docker host: {exc}")

    def get_docker_disk_usage_facts(self) -> dict[str, t.Any]:
        try:
            if self.verbose_output:
                return self.client.df()
            return {"LayersSize": self.client.df()["LayersSize"]}
        except APIError as exc:
            self.client.fail(f"Error inspecting docker host: {exc}")

    def get_docker_items_list(
        self,
        docker_object: str,
        filters: dict[str, t.Any] | None = None,
        verbose: bool = False,
    ) -> list[dict[str, t.Any]]:
        items = []

        header_containers = [
            "Id",
            "Image",
            "Command",
            "Created",
            "Status",
            "Ports",
            "Names",
        ]
        header_volumes = ["Driver", "Name"]
        header_images = ["Id", "RepoTags", "Created", "Size"]
        header_networks = ["Id", "Driver", "Name", "Scope"]

        filter_arg = {}
        if filters:
            filter_arg["filters"] = filters
        try:
            if docker_object == "containers":
                params = {
                    "limit": -1,
                    "all": 1 if self.client.module.params["containers_all"] else 0,
                    "size": 0,
                    "trunc_cmd": 0,
                    "filters": convert_filters(filters) if filters else None,
                }
                items = self.client.get_json("/containers/json", params=params)
            elif docker_object == "networks":
                params = {"filters": convert_filters(filters or {})}
                items = self.client.get_json("/networks", params=params)
            elif docker_object == "images":
                params = {
                    "only_ids": 0,
                    "all": 0,
                    "filters": convert_filters(filters) if filters else None,
                }
                items = self.client.get_json("/images/json", params=params)
            elif docker_object == "volumes":
                params = {
                    "filters": convert_filters(filters) if filters else None,
                }
                items = self.client.get_json("/volumes", params=params)
                items = items["Volumes"]
        except APIError as exc:
            self.client.fail(
                f"Error inspecting docker host for object '{docker_object}': {exc}"
            )

        if self.verbose_output:
            return items

        items_list = []
        for item in items:
            item_record = {}

            if docker_object == "containers":
                for key in header_containers:
                    item_record[key] = item.get(key)
            elif docker_object == "networks":
                for key in header_networks:
                    item_record[key] = item.get(key)
            elif docker_object == "images":
                for key in header_images:
                    item_record[key] = item.get(key)
            elif docker_object == "volumes":
                for key in header_volumes:
                    item_record[key] = item.get(key)
            items_list.append(item_record)

        return items_list


def main() -> None:
    argument_spec = {
        "containers": {"type": "bool", "default": False},
        "containers_all": {"type": "bool", "default": False},
        "containers_filters": {"type": "dict"},
        "images": {"type": "bool", "default": False},
        "images_filters": {"type": "dict"},
        "networks": {"type": "bool", "default": False},
        "networks_filters": {"type": "dict"},
        "volumes": {"type": "bool", "default": False},
        "volumes_filters": {"type": "dict"},
        "disk_usage": {"type": "bool", "default": False},
        "verbose_output": {"type": "bool", "default": False},
    }

    client = AnsibleDockerClient(
        argument_spec=argument_spec,
        supports_check_mode=True,
        fail_results={
            "can_talk_to_docker": False,
        },
    )
    if (
        client.module.params["api_version"] is None
        or client.module.params["api_version"].lower() == "auto"
    ):
        # At this point we know that we can talk to Docker, since we asked it for the API version
        client.fail_results["can_talk_to_docker"] = True

    try:
        results = {
            "changed": False,
        }

        DockerHostManager(client, results)
        client.module.exit_json(**results)
    except DockerException as e:
        client.fail(
            f"An unexpected Docker error occurred: {e}",
            exception=traceback.format_exc(),
        )
    except RequestException as e:
        client.fail(
            f"An unexpected requests error occurred when trying to talk to the Docker daemon: {e}",
            exception=traceback.format_exc(),
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Anvitha Jain (@anvjain) <anvjain@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_macsec_policy
short_description: Manage MACsec Policies on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage MACsec Policies on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.1 (NDO v4.3) and later.
author:
- Anvitha Jain (@anvjain)
options:
  template:
    description:
    - The name of the template.
    - The template must be a fabric policy template.
    type: str
    required: true
  macsec_policy:
    description:
    - The name of the MACsec Policy.
    type: str
    aliases: [ name ]
  macsec_policy_uuid:
    description:
    - The UUID of the MACsec Policy.
    - This parameter is required when the O(macsec_policy) needs to be updated.
    type: str
    aliases: [ uuid ]
  description:
    description:
    - The description of the MACsec Policy.
    type: str
  admin_state:
    description:
    - The administrative state of the MACsec Policy. (Enables or disables the policy)
    - The default value is C(enabled).
    type: str
    choices: [ enabled, disabled ]
  interface_type:
    description:
    - The type of the interfaces this policy will be applied to.
    type: str
    choices: [ fabric, access ]
    default: fabric
  cipher_suite:
    description:
    - The cipher suite to be used for encryption.
    - The default value is C(256_gcm_aes_xpn).
    type: str
    choices: [ 128_gcm_aes, 128_gcm_aes_xpn, 256_gcm_aes, 256_gcm_aes_xpn ]
  window_size:
    description:
    - The window size defines the maximum number of frames that can be received out of order
    - before a replay attack is detected.
    - The value must be between 0 and 4294967295.
    - The default value is 0 for type C(fabric) and 64 for type C(access).
    type: int
  security_policy:
    description:
    - The security policy to allow traffic on the link for the MACsec Policy.
    - The default value is C(should_secure).
    type: str
    choices: [ should_secure, must_secure ]
  sak_expiry_time:
    description:
    - The expiry time for the Security Association Key (SAK) for the MACsec Policy.
    - The value must be 0 or between 60 and 2592000.
    - The default value is 0.
    type: int
  confidentiality_offset:
    description:
    - The confidentiality offset for the MACsec Policy.
    - The default value is 0.
    - This parameter is only available for type C(access).
    type: int
    choices: [ 0, 30, 50 ]
  key_server_priority:
    description:
    - The key server priority for the MACsec Policy.
    - The value must be between 0 and 255.
    - The default value 16 for type C(access).
    - This parameter is only available for type C(access).
    type: int
  macsec_keys:
    description:
    - List of the MACsec Keys.
    - Providing an empty list will remove the O(macsec_keys) from the MACsec Policy.
    - The old O(macsec_keys) entries will be replaced with the new entries during update.
    type: list
    elements: dict
    suboptions:
      key_name:
        description:
        - The name of the MACsec Key.
        - Key Name has to be Hex chars [0-9a-fA-F]
        type: str
        required: true
      psk:
        description:
        - The Pre-Shared Key (PSK) for the MACsec Key.
        - PSK has to be 64 chars long if cipher suite is C(256_gcm_aes) or C(256_gcm_aes_xpn).
        - PSK has to be 32 chars long if cipher suite is C(128_gcm_aes) or C(128_gcm_aes_xpn).
        - PSK has to be Hex chars [0-9a-fA-F]
        type: str
        required: true
      start_time:
        description:
        - The start time for the MACsec Key.
        - The date time format - YYYY-MM-DD HH:MM:SS or 'now'
        - The start time for each key_name should be unique.
        - The default value is C(now).
        type: str
      end_time:
        description:
        - The end time for the MACsec Key.
        - The date time format - YYYY-MM-DD HH:MM:SS or 'infinite'
        - The default value is C(infinite).
        type: str
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a new MACsec Policy of interface_type fabric
  cisco.mso.ndo_macsec_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_template
    macsec_policy: ansible_test_macsec_policy
    description: "Ansible Test MACsec Policy"
    state: present

- name: Create a new MACsec Policy of interface_type access
  cisco.mso.ndo_macsec_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_template
    macsec_policy: ansible_test_macsec_policy
    description: "Ansible Test MACsec Policy"
    macsec_keys:
      - key_name: ansible_test_key
        psk: 'AA111111111111111111111111111111111111111111111111111111111111aa'
        start_time: '2029-12-11 11:12:13'
        end_time: 'infinite'
    state: present

- name: Query a MACsec Policy with macsec_policy name
  cisco.mso.ndo_macsec_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_template
    macsec_policy: ansible_test_macsec_policy
    state: query
  register: query_one

- name: Query all MACsec Policies
  cisco.mso.ndo_macsec_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_template
    state: query
  register: query_all

- name: Query a MACsec Policy with macsec_policy UUID
  cisco.mso.ndo_macsec_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_template
    macsec_policy_uuid: ansible_test_macsec_policy_uuid
    state: query
  register: query_uuid

- name: Delete a MACsec Policy with name
  cisco.mso.ndo_macsec_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_template
    macsec_policy: ansible_test_macsec_policy
    state: absent

- name: Delete a MACsec Policy with UUID
  cisco.mso.ndo_macsec_policy:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_template
    macsec_policy_uuid: ansible_test_macsec_policy_uuid
    state: absent
"""

RETURN = r"""
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate, KVPair
from ansible_collections.cisco.mso.plugins.module_utils.constants import NDO_CIPHER_SUITE_MAP, NDO_SECURITY_POLICY_MAP
import copy


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        dict(
            template=dict(type="str", required=True),
            macsec_policy=dict(type="str", aliases=["name"]),
            macsec_policy_uuid=dict(type="str", aliases=["uuid"]),
            description=dict(type="str"),
            admin_state=dict(type="str", choices=["enabled", "disabled"]),
            interface_type=dict(type="str", choices=["fabric", "access"], default="fabric"),
            cipher_suite=dict(type="str", choices=list(NDO_CIPHER_SUITE_MAP)),
            window_size=dict(type="int"),
            security_policy=dict(type="str", choices=list(NDO_SECURITY_POLICY_MAP)),
            sak_expiry_time=dict(type="int"),
            confidentiality_offset=dict(type="int", choices=[0, 30, 50]),
            key_server_priority=dict(type="int"),
            macsec_keys=dict(
                type="list",
                elements="dict",
                options=dict(
                    key_name=dict(type="str", required=True),
                    psk=dict(type="str", required=True, no_log=True),
                    start_time=dict(type="str"),
                    end_time=dict(type="str"),
                ),
                no_log=False,
            ),
            state=dict(type="str", choices=["absent", "query", "present"], default="query"),
        )
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "present", ["macsec_policy", "macsec_policy_uuid"], True],
            ["state", "absent", ["macsec_policy", "macsec_policy_uuid"], True],
        ],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    macsec_policy = module.params.get("macsec_policy")
    macsec_policy_uuid = module.params.get("macsec_policy_uuid")
    description = module.params.get("description")
    admin_state = module.params.get("admin_state")
    interface_type = module.params.get("interface_type")
    cipher_suite = NDO_CIPHER_SUITE_MAP.get(module.params.get("cipher_suite"))
    window_size = module.params.get("window_size")
    security_policy = NDO_SECURITY_POLICY_MAP.get(module.params.get("security_policy"))
    sak_expiry_time = module.params.get("sak_expiry_time")
    confidentiality_offset = module.params.get("confidentiality_offset")
    key_server_priority = module.params.get("key_server_priority")
    macsec_keys = module.params.get("macsec_keys")
    state = module.params.get("state")

    ops = []
    match = None

    mso_template = MSOTemplate(mso, "fabric_policy", template)
    mso_template.validate_template("fabricPolicy")

    path = "/fabricPolicyTemplate/template/macsecPolicies"
    object_description = "MACsec Policy"

    existing_macsec_policies = mso_template.template.get("fabricPolicyTemplate", {}).get("template", {}).get("macsecPolicies", [])
    if macsec_policy or macsec_policy_uuid:
        match = mso_template.get_object_by_key_value_pairs(
            object_description,
            existing_macsec_policies,
            [KVPair("uuid", macsec_policy_uuid) if macsec_policy_uuid else KVPair("name", macsec_policy)],
        )
        if match:
            mso.existing = mso.previous = copy.deepcopy(match.details)
    else:
        mso.existing = mso.previous = existing_macsec_policies

    if state == "present":

        if match:

            if macsec_policy and match.details.get("name") != macsec_policy:
                ops.append(dict(op="replace", path="{0}/{1}/name".format(path, match.index), value=macsec_policy))
                match.details["name"] = macsec_policy

            if description is not None and match.details.get("description") != description:
                ops.append(dict(op="replace", path="{0}/{1}/description".format(path, match.index), value=description))
                match.details["description"] = description

            if admin_state and match.details.get("adminState") != admin_state:
                ops.append(dict(op="replace", path="{0}/{1}/adminState".format(path, match.index), value=admin_state))
                match.details["adminState"] = admin_state

            if interface_type and match.details.get("type") != interface_type:
                mso.fail_json(msg="Type cannot be changed for an existing MACsec Policy.")

            if cipher_suite and match.details.get("macsecParams")["cipherSuite"] != cipher_suite:
                ops.append(dict(op="replace", path="{0}/{1}/macsecParams/cipherSuite".format(path, match.index), value=cipher_suite))
                match.details["macsecParams"]["cipherSuite"] = cipher_suite

            if window_size and match.details.get("macsecParams")["windowSize"] != window_size:
                ops.append(dict(op="replace", path="{0}/{1}/macsecParams/windowSize".format(path, match.index), value=window_size))
                match.details["macsecParams"]["windowSize"] = window_size

            if security_policy and match.details.get("macsecParams")["securityPol"] != security_policy:
                ops.append(dict(op="replace", path="{0}/{1}/macsecParams/securityPol".format(path, match.index), value=security_policy))
                match.details["macsecParams"]["securityPol"] = security_policy

            if sak_expiry_time and match.details.get("macsecParams")["sakExpiryTime"] != sak_expiry_time:
                ops.append(dict(op="replace", path="{0}/{1}/macsecParams/sakExpiryTime".format(path, match.index), value=sak_expiry_time))
                match.details["macsecParams"]["sakExpiryTime"] = sak_expiry_time

            if interface_type == "access":
                if confidentiality_offset and match.details.get("macsecParams")["confOffSet"] != confidentiality_offset:
                    ops.append(
                        dict(op="replace", path="{0}/{1}/macsecParams/confOffSet".format(path, match.index), value="offset{0}".format(confidentiality_offset))
                    )
                    match.details["macsecParams"]["confOffSet"] = "offset{0}".format(confidentiality_offset)

                if key_server_priority and match.details.get("macsecParams")["keyServerPrio"] != key_server_priority:
                    ops.append(dict(op="replace", path="{0}/{1}/macsecParams/keyServerPrio".format(path, match.index), value=key_server_priority))
                    match.details["macsecParams"]["keyServerPrio"] = key_server_priority

            if macsec_keys:
                # updating macsec_keys modifies the existing list with the new list
                macsec_keys_list = []
                for macsec_key in macsec_keys:
                    macsec_keys_list.append(
                        dict(
                            keyname=macsec_key.get("key_name"),
                            psk=macsec_key.get("psk"),
                            start=mso.verify_time_format(macsec_key.get("start_time")) if macsec_key.get("start_time") else None,
                            end=mso.verify_time_format(macsec_key.get("end_time")) if macsec_key.get("end_time") else None,
                        )
                    )

                if macsec_keys_list != match.details.get("macsecKeys", []):
                    ops.append(dict(op="replace", path="{0}/{1}/macsecKeys".format(path, match.index), value=macsec_keys_list))
                match.details["macsecKeys"] = macsec_keys
            elif macsec_keys == []:
                # remove macsec_keys if the list is empty
                ops.append(dict(op="remove", path="{0}/{1}/macsecKeys".format(path, match.index)))
                match.details.pop("macsecKeys", None)

            mso.sanitize(match.details)

        else:
            macsec_param_map = {}

            payload = {"name": macsec_policy, "templateId": mso_template.template.get("templateId"), "schemaId": mso_template.template.get("schemaId")}
            payload["type"] = interface_type

            if description:
                payload["description"] = description
            if admin_state:
                payload["adminState"] = admin_state
            if cipher_suite:
                macsec_param_map["cipherSuite"] = cipher_suite
            if window_size:
                macsec_param_map["windowSize"] = window_size
            if security_policy:
                macsec_param_map["securityPol"] = security_policy
            if sak_expiry_time:
                macsec_param_map["sakExpiryTime"] = sak_expiry_time

            if interface_type == "access":
                if confidentiality_offset:
                    macsec_param_map["confOffSet"] = "offset{0}".format(confidentiality_offset)
                if key_server_priority:
                    macsec_param_map["keyServerPrio"] = key_server_priority
                payload["macsecParams"] = macsec_param_map

            if macsec_keys:
                macsec_keys_list = []
                for macsec_key in macsec_keys:
                    macsec_key_dict = {
                        "keyname": macsec_key.get("key_name"),
                        "psk": macsec_key.get("psk"),
                    }
                    if macsec_key.get("start_time"):
                        macsec_key_dict["start"] = macsec_key.get("start_time")
                    if macsec_key.get("end_time"):
                        macsec_key_dict["end"] = macsec_key.get("end_time")
                    macsec_keys_list.append(macsec_key_dict)
                payload["macsecKeys"] = macsec_keys_list

            ops.append(dict(op="add", path="{0}/-".format(path), value=copy.deepcopy(payload)))

            mso.sanitize(payload)

        mso.existing = mso.proposed

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path="{0}/{1}".format(path, match.index)))

    if not module.check_mode and ops:
        response = mso.request(mso_template.template_path, method="PATCH", data=ops)
        macsec_policies = response.get("fabricPolicyTemplate", {}).get("template", {}).get("macsecPolicies", [])
        match = mso_template.get_object_by_key_value_pairs(
            object_description,
            macsec_policies,
            [KVPair("uuid", macsec_policy_uuid) if macsec_policy_uuid else KVPair("name", macsec_policy)],
        )
        if match:
            mso.existing = match.details
        else:
            mso.existing = {}
    elif module.check_mode and state != "query":
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# cli_rm_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the module docstring and re-run
# cli_rm_builder.
#
#############################################

"""
The arg spec for the nxos_route_maps module
"""


class Route_mapsArgs(object):  # pylint: disable=R0903
    """The arg spec for the nxos_route_maps module"""

    argument_spec = {
        "running_config": {"type": "str"},
        "config": {
            "type": "list",
            "elements": "dict",
            "options": {
                "route_map": {"type": "str"},
                "entries": {
                    "type": "list",
                    "elements": "dict",
                    "options": {
                        "sequence": {"type": "int"},
                        "action": {
                            "type": "str",
                            "choices": ["deny", "permit"],
                        },
                        "continue_sequence": {"type": "int"},
                        "description": {"type": "str"},
                        "match": {
                            "type": "dict",
                            "options": {
                                "as_number": {
                                    "type": "dict",
                                    "options": {
                                        "asn": {
                                            "type": "list",
                                            "elements": "str",
                                        },
                                        "as_path_list": {
                                            "type": "list",
                                            "elements": "str",
                                        },
                                    },
                                },
                                "as_path": {"type": "list", "elements": "str"},
                                "community": {
                                    "type": "dict",
                                    "options": {
                                        "community_list": {
                                            "type": "list",
                                            "elements": "str",
                                        },
                                        "exact_match": {"type": "bool"},
                                    },
                                },
                                "evpn": {
                                    "type": "dict",
                                    "options": {
                                        "route_types": {
                                            "type": "list",
                                            "elements": "str",
                                        },
                                    },
                                },
                                "extcommunity": {
                                    "type": "dict",
                                    "options": {
                                        "extcommunity_list": {
                                            "type": "list",
                                            "elements": "str",
                                        },
                                        "exact_match": {"type": "bool"},
                                    },
                                },
                                "interfaces": {
                                    "type": "list",
                                    "elements": "str",
                                },
                                "ip": {
                                    "type": "dict",
                                    "options": {
                                        "address": {
                                            "type": "dict",
                                            "options": {
                                                "access_list": {"type": "str"},
                                                "prefix_lists": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                            },
                                        },
                                        "multicast": {
                                            "type": "dict",
                                            "options": {
                                                "source": {"type": "str"},
                                                "group": {
                                                    "type": "dict",
                                                    "options": {"prefix": {"type": "str"}},
                                                },
                                                "group_range": {
                                                    "type": "dict",
                                                    "options": {
                                                        "first": {"type": "str"},
                                                        "last": {"type": "str"},
                                                    },
                                                },
                                                "rp": {
                                                    "type": "dict",
                                                    "options": {
                                                        "prefix": {"type": "str"},
                                                        "rp_type": {
                                                            "type": "str",
                                                            "choices": [
                                                                "ASM",
                                                                "Bidir",
                                                            ],
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                        "next_hop": {
                                            "type": "dict",
                                            "options": {
                                                "prefix_lists": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                            },
                                        },
                                        "route_source": {
                                            "type": "dict",
                                            "options": {
                                                "prefix_lists": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                            },
                                        },
                                    },
                                },
                                "ipv6": {
                                    "type": "dict",
                                    "options": {
                                        "address": {
                                            "type": "dict",
                                            "options": {
                                                "access_list": {"type": "str"},
                                                "prefix_lists": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                            },
                                        },
                                        "multicast": {
                                            "type": "dict",
                                            "options": {
                                                "source": {"type": "str"},
                                                "group": {
                                                    "type": "dict",
                                                    "options": {"prefix": {"type": "str"}},
                                                },
                                                "group_range": {
                                                    "type": "dict",
                                                    "options": {
                                                        "first": {"type": "str"},
                                                        "last": {"type": "str"},
                                                    },
                                                },
                                                "rp": {
                                                    "type": "dict",
                                                    "options": {
                                                        "prefix": {"type": "str"},
                                                        "rp_type": {
                                                            "type": "str",
                                                            "choices": [
                                                                "ASM",
                                                                "Bidir",
                                                            ],
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                        "next_hop": {
                                            "type": "dict",
                                            "options": {
                                                "prefix_lists": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                            },
                                        },
                                        "route_source": {
                                            "type": "dict",
                                            "options": {
                                                "prefix_lists": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                            },
                                        },
                                    },
                                },
                                "mac_list": {
                                    "type": "list",
                                    "elements": "str",
                                },
                                "metric": {"type": "list", "elements": "int"},
                                "ospf_area": {
                                    "type": "list",
                                    "elements": "int",
                                },
                                "route_types": {
                                    "type": "list",
                                    "elements": "str",
                                    "choices": [
                                        "external",
                                        "inter-area",
                                        "internal",
                                        "intra-area",
                                        "level-1",
                                        "level-2",
                                        "local",
                                        "nssa-external",
                                        "type-1",
                                        "type-2",
                                    ],
                                },
                                "source_protocol": {
                                    "type": "list",
                                    "elements": "str",
                                },
                                "tags": {"type": "list", "elements": "int"},
                            },
                        },
                        "set": {
                            "type": "dict",
                            "options": {
                                "as_path": {
                                    "type": "dict",
                                    "options": {
                                        "prepend": {
                                            "type": "dict",
                                            "options": {
                                                "as_number": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                                "last_as": {"type": "int"},
                                            },
                                        },
                                        "tag": {"type": "bool"},
                                    },
                                },
                                "comm_list": {"type": "str"},
                                "community": {
                                    "type": "dict",
                                    "options": {
                                        "additive": {"type": "bool"},
                                        "graceful_shutdown": {"type": "bool"},
                                        "internet": {"type": "bool"},
                                        "local_as": {"type": "bool"},
                                        "no_advertise": {"type": "bool"},
                                        "no_export": {"type": "bool"},
                                        "number": {
                                            "type": "list",
                                            "elements": "str",
                                        },
                                    },
                                },
                                "dampening": {
                                    "type": "dict",
                                    "options": {
                                        "half_life": {"type": "int"},
                                        "start_reuse_route": {"type": "int"},
                                        "start_suppress_route": {"type": "int"},
                                        "max_suppress_time": {"type": "int"},
                                    },
                                },
                                "distance": {
                                    "type": "dict",
                                    "options": {
                                        "igp_ebgp_routes": {"type": "int"},
                                        "internal_routes": {"type": "int"},
                                        "local_routes": {"type": "int"},
                                    },
                                },
                                "evpn": {
                                    "type": "dict",
                                    "options": {
                                        "gateway_ip": {
                                            "type": "dict",
                                            "mutually_exclusive": [["ip", "use_nexthop"]],
                                            "options": {
                                                "ip": {"type": "str"},
                                                "use_nexthop": {"type": "bool"},
                                            },
                                        },
                                    },
                                },
                                "extcomm_list": {"type": "str"},
                                "extcommunity": {
                                    "type": "dict",
                                    "options": {
                                        "rt": {
                                            "type": "dict",
                                            "options": {
                                                "additive": {"type": "bool"},
                                                "extcommunity_numbers": {
                                                    "type": "list",
                                                    "elements": "str",
                                                },
                                            },
                                        },
                                    },
                                },
                                "forwarding_address": {"type": "bool"},
                                "null_interface": {"type": "str"},
                                "ip": {
                                    "type": "dict",
                                    "options": {
                                        "address": {
                                            "type": "dict",
                                            "options": {"prefix_list": {"type": "str"}},
                                        },
                                        "precedence": {"type": "str"},
                                        "next_hop": {
                                            "type": "dict",
                                            "mutually_exclusive": [
                                                [
                                                    "address",
                                                    "peer_address",
                                                    "redist_unchanged",
                                                    "verify_availability",
                                                ],
                                                [
                                                    "address",
                                                    "peer_address",
                                                    "unchanged",
                                                    "verify_availability",
                                                ],
                                            ],
                                            "options": {
                                                "address": {"type": "str"},
                                                "drop_on_fail": {"type": "bool", "default": False},
                                                "force_order": {"type": "bool", "default": False},
                                                "load_share": {"type": "bool", "default": False},
                                                "peer_address": {"type": "bool"},
                                                "redist_unchanged": {"type": "bool"},
                                                "unchanged": {"type": "bool"},
                                                "verify_availability": {
                                                    "type": "list",
                                                    "elements": "dict",
                                                    "options": {
                                                        "address": {
                                                            "type": "str",
                                                            "required": True,
                                                        },
                                                        "track": {"type": "int", "required": True},
                                                        "drop_on_fail": {
                                                            "type": "bool",
                                                            "default": False,
                                                        },
                                                        "force_order": {
                                                            "type": "bool",
                                                            "default": False,
                                                        },
                                                        "load_share": {
                                                            "type": "bool",
                                                            "default": False,
                                                        },
                                                    },
                                                },
                                            },
                                        },
                                    },
                                },
                                "ipv6": {
                                    "type": "dict",
                                    "options": {
                                        "address": {
                                            "type": "dict",
                                            "options": {"prefix_list": {"type": "str"}},
                                        },
                                        "precedence": {"type": "str"},
                                    },
                                },
                                "label_index": {"type": "int"},
                                "level": {
                                    "type": "str",
                                    "choices": [
                                        "level-1",
                                        "level-1-2",
                                        "level-2",
                                    ],
                                },
                                "local_preference": {"type": "int"},
                                "metric": {
                                    "type": "dict",
                                    "options": {
                                        "bandwidth": {"type": "int"},
                                        "igrp_delay_metric": {"type": "int"},
                                        "igrp_reliability_metric": {"type": "int"},
                                        "igrp_effective_bandwidth_metric": {"type": "int"},
                                        "igrp_mtu": {"type": "int"},
                                    },
                                },
                                "metric_type": {
                                    "type": "str",
                                    "choices": [
                                        "external",
                                        "internal",
                                        "type-1",
                                        "type-2",
                                    ],
                                },
                                "nssa_only": {"type": "bool"},
                                "origin": {
                                    "type": "str",
                                    "choices": ["egp", "igp", "incomplete"],
                                },
                                "path_selection": {
                                    "type": "str",
                                    "choices": [
                                        "all",
                                        "backup",
                                        "best2",
                                        "multipaths",
                                    ],
                                },
                                "tag": {"type": "int"},
                                "weight": {"type": "int"},
                            },
                        },
                    },
                },
            },
        },
        "state": {
            "type": "str",
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "parsed",
                "gathered",
                "rendered",
            ],
            "default": "merged",
        },
    }  # pylint: disable=C0301



#!/usr/bin/env python

import yaml
import os

github_workflow_file = '.github/workflows/ansible-test-plugins.yml'


def read_github_workflow_file():
    with open(github_workflow_file, 'r') as gh_file:
        try:
            return yaml.safe_load(gh_file)
        except yaml.YAMLError as exc:
            print(exc)


def extract_value(target, dict_yaml):
    for key, value in dict_yaml.items():
        if key == target:
            return value


def extract_matrix(workflow_yaml):
    jobs = extract_value('jobs', workflow_yaml)
    integration = extract_value('integration', jobs)
    strategy = extract_value('strategy', integration)
    matrix = extract_value('matrix', strategy)
    return matrix


def is_exclude(exclude_list, test_suite):
    test_is_excluded = False
    for excl in exclude_list:
        match = 0

        if 'ansible' in excl:
            if excl.get('ansible') == test_suite.get('ansible'):
                match += 1

        if 'db_engine_name' in excl:
            if excl.get('db_engine_name') == test_suite.get('db_engine_name'):
                match += 1

        if 'db_engine_version' in excl:
            if excl.get('db_engine_version') == test_suite.get('db_engine_version'):
                match += 1

        if 'python' in excl:
            if excl.get('python') == test_suite.get('python'):
                match += 1

        if 'connector_name' in excl:
            if excl.get('connector_name') == test_suite.get('connector_name'):
                match += 1

        if 'connector_version' in excl:
            if excl.get('connector_version') == test_suite.get('connector_version'):
                match += 1

        if match > 1:
            test_is_excluded = True
            return test_is_excluded

    return test_is_excluded


def main():
    workflow_yaml = read_github_workflow_file()
    tests_matrix_yaml = extract_matrix(workflow_yaml)

    matrix = []
    exclude_list = tests_matrix_yaml.get('exclude')
    for ansible in tests_matrix_yaml.get('ansible'):
        for db_engine_name in tests_matrix_yaml.get('db_engine_name'):
            for db_engine_version in tests_matrix_yaml.get('db_engine_version'):
                for python in tests_matrix_yaml.get('python'):
                    for connector_name in tests_matrix_yaml.get('connector_name'):
                        for connector_version in tests_matrix_yaml.get('connector_version'):
                            test_suite = {
                                'ansible': ansible,
                                'db_engine_name': db_engine_name,
                                'db_engine_version': db_engine_version,
                                'python': python,
                                'connector_name': connector_name,
                                'connector_version': connector_version
                            }
                            if not is_exclude(exclude_list, test_suite):
                                matrix.append(test_suite)

    for tests in matrix:
        a = tests.get('ansible')
        dn = tests.get('db_engine_name')
        dv = tests.get('db_engine_version')
        p = tests.get('python')
        cn = tests.get('connector_name')
        cv = tests.get('connector_version')
        make_cmd = (
            f'make '
            f'ansible="{a}" '
            f'db_engine_name="{dn}" '
            f'db_engine_version="{dv}" '
            f'python="{p}" '
            f'connector_name="{cn}" '
            f'connector_version="{cv}" '
            f'test-integration'
        )
        print(f'Run tests for: Ansible: {a}, DB: {dn} {dv}, Python: {p}, Connector: {cn} {cv}')
        os.system(make_cmd)
        # TODO, allow for CTRL+C to break the loop more easily
        # TODO, store the failures from this iteration
    # TODO, display a summary of failures from every iterations


if __name__ == '__main__':
    main()
